// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * Ce package est le controleur des &eacute;tats du jeu.
 */
package controller;

import java.awt.Point;
import java.util.ArrayList;

import model.characters.Hero;

/**
 * Cette classe traitera toutes les collisions du jeu
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-16, 12:15pm <br>
 *          0.1.2 : 2011-01-19, 8:31pm <br>
 *          0.1.3 : 2011-01-19, 9:44pm
 */
public class Collision {

	////////////////////////
	// Méthodes statiques //
	////////////////////////
	/**
	 * Test si le joueur peut passer ou pas
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @param blocked
	 *            La liste des &eacute;l&eacute;ments bloquant de la carte
	 * @return Vrai si le h&eacute;ro est bloqu&eacute; (Faux sinon)
	 */
	public static boolean collisionTiles (Hero box, ArrayList<Point> blocked) {
		int i1 = box.getX () / 32;
		int j1 = box.getY () / 32;
		int i2 = (box.getX () + 24 - 1) / 32;
		int j2 = (box.getY () + 16 - 1) / 32;
		int i, j;
		for (i = i1; i <= i2; i++) {
			for (j = j1; j <= j2; j++) {
				if (tileIsBlocking (i, j, blocked)) return true;
			}
		}
		return false; // si on n'est pas sorti avant, c'est qu'on ne touche aucun tile.
	}

	/**
	 * Test si le joueur passe lentement ou pas
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @param slowier
	 *            La liste des &eacute;l&eacute;ments ralentissant de la carte
	 * @return Vrai si le h&eacute;ro est ralenti (Faux sinon)
	 */
	public static boolean collisionSlowlyTiles (Hero box, ArrayList<Point> slowier) {
		int i1 = box.getX () / 32;
		int j1 = box.getY () / 32;
		int i2 = (box.getX () + 24 - 1) / 32;
		int j2 = (box.getY () + 16 - 1) / 32;
		int i, j;
		for (i = i1; i <= i2; i++) {
			for (j = j1; j <= j2; j++) {
				if (tileIsSlowing (i, j, slowier)) return true;
			}
		}
		return false; // si on n'est pas sorti avant, c'est qu'on ne touche aucun tile.
	}

	/**
	 * Teste si le tile est bloquant
	 * 
	 * @param i
	 *            L'abscisse du joueur
	 * @param j
	 *            L'ordon&eacute;e du joueur
	 * @param blocked
	 *            La liste des &eacute;l&eacute;ments bloquant de la map
	 * @return Vrai si la case est un tile bloquant (Faux sinon)
	 */
	private static boolean tileIsBlocking (int i, int j, ArrayList<Point> blocked) {
		if (blocked.contains (new Point (i, j))) return true;
		else return false;
	}

	/**
	 * Teste si le tile est ralentissant
	 * 
	 * @param i
	 *            L'abscisse du joueur
	 * @param j
	 *            L'ordon&eacute;e du joueur
	 * @param slowier
	 *            La liste des &eacute;l&eacute;ments ralentisseur de la map
	 * @return Vrai si la case est un tile ralentissant (Faux sinon)
	 */
	private static boolean tileIsSlowing (int i, int j, ArrayList<Point> slowier) {
		if (slowier.contains (new Point (i, j))) return true;
		else return false;
	}

	/**
	 * Permet de savoir si le joueur a atteint le bord haut de la map
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @return Vrai si c'est le bord de la map (Faux sinon)
	 */
	public static boolean collisionMapUpEdge (Hero box) {
		if (box.getY () < 0) return true;
		else return false;
	}

	/**
	 * Permet de savoir si le joueur a atteint le bord bas de la map
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @return Vrai si c'est le bord de la map (Faux sinon)
	 */
	public static boolean collisionMapDownEdge (Hero box) {
		if (box.getY () > 576) return true;
		else return false;
	}

	/**
	 * Permet de savoir si le joueur a atteint le bord gauche de la map
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @return Vrai si c'est le bord de la map (Faux sinon)
	 */
	public static boolean collisionMapLeftEdge (Hero box) {
		if (box.getX () < 0) return true;
		else return false;
	}

	/**
	 * Permet de savoir si le joueur a atteint le bord droit de la map
	 * 
	 * @param box
	 *            Le h&eacute;ro
	 * @return Vrai si c'est le bord de la map (Faux sinon)
	 */
	public static boolean collisionMapRightEdge (Hero box) {
		if (box.getX () > 800) return true;
		else return false;
	}
}
