// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * C'est le controller du jeu
 */
package controller;

import loader.events.EventXMLLoader;
import model.Model;
import model.SaveLoad;
import model.menu.BattleMenu;
import model.menu.InformationMenu;
import model.menu.NpcMenu;
import model.menu.SaveMenu;

import org.newdawn.slick.Input;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

import enumeration.Moving;

/**
 * <code>Controller</code> s'occupe de toute la partie control des intéractions
 * clavier-souris
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-17, 8:30pm <br>
 *          0.1.2 : 2011-01-19, 8:32pm <br>
 *          0.1.3 : 2011-01-19, 9:44pm <br>
 */
public class Controller {

	///////////////
	// Attributs //
	///////////////
	/**
	 * Model du jeu
	 */
	private Model model;
	/////////////////////////////
	// touches pressées ou non //
	/////////////////////////////
	/**
	 * Touche du bas pressée
	 */
	private boolean releasedDownKey = true;
	/**
	 * Touche vers la gauche pressée
	 */
	private boolean releasedLeftKey = true;
	/**
	 * Touche vers la droite pressée
	 */
	private boolean releasedRightKey = true;
	/**
	 * Touche vers le haut pressée
	 */
	private boolean releasedUpKey = true;

	private StateBasedGame sbg;

	//////////////////
	// Constructeur //
	//////////////////
	/**
	 * Fixe le model du jeu
	 * 
	 * @param model
	 *            Le modèle
	 */
	public Controller (Model model) {
		this.setModel (model);
	}

	//////////////
	// Méthodes //
	//////////////
	/**
	 * Récupère toutes les actions du clavier
	 * 
	 * @param input
	 *            le clavier
	 */
	public void controlInput (Input input, StateBasedGame sbg) {
		this.sbg = sbg;
		if (!model.isOutMap ()) { // Sur la map
			// Partie déplacement du joueur
			if (input.isKeyDown (Input.KEY_UP)) this.upMoving ();
			else if (input.isKeyDown (Input.KEY_DOWN)) this.downMoving ();
			else if (input.isKeyDown (Input.KEY_LEFT)) this.leftMoving ();
			else if (input.isKeyDown (Input.KEY_RIGHT)) this.rightMoving ();
			else if ((input.isKeyPressed (Input.KEY_ENTER)) || (input.isKeyPressed (Input.KEY_SPACE))) this.enterEvent ();
			else if (input.isKeyPressed (Input.KEY_ESCAPE)) this.enterMenuGlobal (sbg);
			else if (input.isKeyPressed (Input.KEY_F)) this.graphicsBattleEngine (sbg);
		}
		else { // Dans un menu
			if ((input.isKeyPressed (Input.KEY_ENTER)) || (input.isKeyPressed (Input.KEY_SPACE))) {
				if (model.isOutMap ()) {
					if (model.getMenu () instanceof NpcMenu) {
						((NpcMenu) model.getMenu ()).setSee (false);
						((NpcMenu) model.getMenu ()).setIncrementable (true);
						if (((NpcMenu) model.getMenu ()).getInc () < ((NpcMenu) model.getMenu ()).getListMessage ().size ()) {
							if ((((NpcMenu) model.getMenu ()).getInc () + 1) < ((NpcMenu) model.getMenu ()).getListMessage ().size ()) ((NpcMenu) model.getMenu ()).setInc (((NpcMenu) model.getMenu ()).getInc () + 1);
							else model.setOutMap (false);
						}
						else model.setOutMap (false);
					}
					else if (model.getMenu () instanceof InformationMenu) {
						model.setOutMap (false);
					}
					else if (model.getMenu () instanceof SaveMenu) {
						if (((SaveMenu) model.getMenu ()).isLifePoint ()) {
							model.getHero ().setHp (model.getHero ().getMaxHP ());
							((SaveMenu) model.getMenu ()).setLifePoint (false);
						}
						else {
							if (!((SaveMenu) model.getMenu ()).isSaving ()) {
								if (((SaveMenu) model.getMenu ()).isSave ()) {
									if (((SaveMenu) model.getMenu ()).isSaveFile ()) {
										this.saveGame ();
										((SaveMenu) model.getMenu ()).setSaving (true);
									}
									else {
										((SaveMenu) model.getMenu ()).setSaveFile (true);
									}
								}
								else {
									model.setOutMap (false);
								}
							}
							else {
								model.setOutMap (false);
							}
						}
					}
					else if (model.getMenu () instanceof BattleMenu) {
						if (((BattleMenu) model.getMenu ()).isCharSelectionned ()) {
							((BattleMenu) model.getMenu ()).setCharSelectionned (false);
						}
						else {
							if (((BattleMenu) model.getMenu ()).isEndFight ()) {
								if (((BattleMenu) model.getMenu ()).isAffichMWin ()) {
									try {
										System.out.println ("#");
										Thread.sleep (2000);
										System.out.println ("#bis");
									}
									catch (InterruptedException e) {
										e.printStackTrace ();
									}
									model.setOutMap (false);
								}
							}
							else {
								((BattleMenu) model.getMenu ()).attack ();
							}
						}
					}
				}
			}
			else if (input.isKeyPressed (Input.KEY_UP)) {
				if (model.isOutMap ()) {
					if (model.getMenu () instanceof SaveMenu) {
						if (!((SaveMenu) model.getMenu ()).isSaveFile ()) {
							if (!((SaveMenu) model.getMenu ()).isSave ()) ((SaveMenu) model.getMenu ()).setSave (true);
						}
						else {
							((SaveMenu) model.getMenu ()).cursorUp ();
						}
					}
					else if (model.getMenu () instanceof BattleMenu) {
						((BattleMenu) model.getMenu ()).upKey ();
					}
				}
			}
			else if (input.isKeyPressed (Input.KEY_DOWN)) {
				if (model.isOutMap ()) {
					if (model.getMenu () instanceof SaveMenu) {
						if (!((SaveMenu) model.getMenu ()).isSaveFile ()) {
							if (((SaveMenu) model.getMenu ()).isSave ()) ((SaveMenu) model.getMenu ()).setSave (false);
						}
						else {
							((SaveMenu) model.getMenu ()).cursorDown ();
						}
					}
					else if (model.getMenu () instanceof BattleMenu) {
						((BattleMenu) model.getMenu ()).downKey ();
					}
				}
			}
			else if (input.isKeyPressed (Input.KEY_ESCAPE)) {
				if (model.isOutMap ()) {
					if (model.getMenu () instanceof SaveMenu) {
						if (((SaveMenu) model.getMenu ()).isSaveFile ()) ((SaveMenu) model.getMenu ()).setSaveFile (false);
					}
					else if (model.getMenu () instanceof BattleMenu) {
						if (((BattleMenu) model.getMenu ()).isCharSelectionned ()) {
							// model.setOutMap (false); // à enlever après la phase de test
						}
						else {
							((BattleMenu) model.getMenu ()).setCharSelectionned (true);
							((BattleMenu) model.getMenu ()).setCursorSkill (0);
						}
					}
				}
			}
		}

		this.releasedKey (input); // gestion des touches relachées

		// Partie animation (si aucune touche directionnelle n'est activé alors on arrête l'animation du joueur)
		if (this.isReleasedDownKey () && this.isReleasedUpKey () && this.isReleasedLeftKey () && this.isReleasedRightKey ()) {
			model.getHero ().pauseAnimation ();
		}
	}

	//////////////////////////////////
	// Partie monstre aléatoire pop //
	//////////////////////////////////
	public void fight () {
		if (!this.getModel ().getMap ().getFightZone ().equalsIgnoreCase ("false") && (this.getModel ().getHero ().getNbStep () > this.getModel ().getTIMEMONSTER ())) {
			this.graphicsBattleEngine (sbg);
			this.getModel ().getHero ().setNbStep (0);
		}
		if (this.getModel ().getHero ().getNbStep () > 600) this.getModel ().getHero ().setNbStep (0);
	}

	//////////////////////////////////
	// Partie moteur des évènements //
	//////////////////////////////////
	/**
	 * Évènement déclenché lors de l'appuie sur la touche "entrée"
	 */
	private void enterEvent () {
		String key = EventCollision.collisionEvent (model.getHero (), model.getMap ().getEventPoint ());
		if (!key.equals ("")) {
			if (EventXMLLoader.getHashMapEvent ().get (key).getEventType ().equalsIgnoreCase ("portal")) {
				model.comeInHouse (key);
			}
			if (EventXMLLoader.getHashMapEvent ().get (key).getEventType ().equalsIgnoreCase ("Information")) {
				model.setOutMap (true);
				model.setMenu (new InformationMenu (key, this.getModel ().getGameVar (), this));
			}
			else if (EventXMLLoader.getHashMapEvent ().get (key).getEventType ().equalsIgnoreCase ("npcS") || EventXMLLoader.getHashMapEvent ().get (key).getEventType ().equalsIgnoreCase ("npcD")) {
				if (EventXMLLoader.getHashMapEvent ().get (key).isActived ()) {
					model.setOutMap (true);
					if (key.equalsIgnoreCase ("n3000-3000-1")) {
						model.setMenu (new SaveMenu (key, this.getModel ().getGameVar (), this));
					}
					else {
						model.setMenu (new NpcMenu (key, this.getModel ().getGameVar (), this));
					}
				}
			}
		}
	}

	private void saveGame () {
		SaveLoad.saveGame (this.getModel ().getGameVar (), this.getModel ().getMap ().getKey (), this.getModel ().getHero ().getX (), this.getModel ().getHero ().getY (), this.getModel ().getHero ().getName () + ";" + ((SaveMenu) model.getMenu ()).getFileSave ()[((SaveMenu) model.getMenu ()).getCursor ()]);
	}

	private boolean onEvent () {
		String key;
		if (!model.getHero ().isOnMap ()) {
			if (!(key = EventCollision.collisionEventOn (model.getHero (), model.getMap ().getEventPoint ())).equals ("")) {
				if (EventXMLLoader.getHashMapEvent ().get (key).getEventType ().equalsIgnoreCase ("portal")) {
					model.comeOutHouse (key);
					return true;
				}
			}
		}
		return false;
	}

	// partie changement d'états //
	private void enterMenuGlobal (StateBasedGame sbg) {
		sbg.enterState (2, new FadeOutTransition (), new FadeInTransition ());
	}

	////////////////////////////////////////////
	// Partie graphique du moteur des combats //
	////////////////////////////////////////////
	/**
	 * 
	 */
	private void graphicsBattleEngine (StateBasedGame sbg) {
		model.setMenu (new BattleMenu (model.getGameVar (), this, sbg));
		model.setOutMap (true);
	}

	////////////////////////////////////////////////////
	// Partie moteur de déplacement et des collisions //
	////////////////////////////////////////////////////
	/**
	 * D&eacute;placement vers le haut
	 */
	private void upMoving () {
		this.setReleasedUpKey (false);
		model.getHero ().setY (model.getHero ().getY () - 4);
		if (Collision.collisionTiles (model.getHero (), model.getMap ().getBlocked ())) {
			if (!this.onEvent ()) {
				if (model.getHero ().isWalking ()) model.getHero ().setWalking (false);
				model.getHero ().setY (model.getHero ().getY () + 4);
				model.getHero ().restartAnimation (Moving.UP);
			}
		}
		else {
			if (!model.getHero ().isWalking ()) model.getHero ().setWalking (true);
			if (Collision.collisionMapUpEdge (model.getHero ())) {
				if (this.isNextMap (0, -1)) this.getModel ().changeMap (this.nextMapKey (0, -1), Moving.UP);
				else {
					model.getHero ().setY (model.getHero ().getY () + 4);
				}
			}
			else {
				if (model.getHero ().getCurrentDirection () != Moving.UP.getDirection ()) model.getHero ().restartAnimation (Moving.UP);
				if (Collision.collisionSlowlyTiles (model.getHero (), model.getMap ().getSlowlier ())) {
					model.getHero ().setY (model.getHero ().getY () + 2);
				}
			}
		}
	}

	/**
	 * D&eacute;placement vers le bas
	 */
	private void downMoving () {
		this.setReleasedDownKey (false);
		model.getHero ().setY (model.getHero ().getY () + 4);
		if (Collision.collisionTiles (model.getHero (), model.getMap ().getBlocked ())) {
			if (!this.onEvent ()) {
				if (model.getHero ().isWalking ()) model.getHero ().setWalking (false);
				model.getHero ().setY (model.getHero ().getY () - 4);
				model.getHero ().restartAnimation (Moving.DOWN);
			}
		}
		else {
			if (!model.getHero ().isWalking ()) model.getHero ().setWalking (true);
			if (Collision.collisionMapDownEdge (model.getHero ())) {
				if (this.isNextMap (0, 1)) this.getModel ().changeMap (this.nextMapKey (0, 1), Moving.DOWN);
				else {
					model.getHero ().setY (model.getHero ().getY () - 4);
				}
			}
			else {
				if (model.getHero ().getCurrentDirection () != Moving.DOWN.getDirection ()) model.getHero ().restartAnimation (Moving.DOWN);
				if (Collision.collisionSlowlyTiles (model.getHero (), model.getMap ().getSlowlier ())) {
					model.getHero ().setY (model.getHero ().getY () - 2);
				}
			}
		}
	}

	/**
	 * D&eacute;placement vers la droite
	 */
	private void rightMoving () {
		this.setReleasedRightKey (false);
		model.getHero ().setX (model.getHero ().getX () + 4);
		if (Collision.collisionTiles (model.getHero (), model.getMap ().getBlocked ())) {
			if (!this.onEvent ()) {
				if (model.getHero ().isWalking ()) model.getHero ().setWalking (false);
				model.getHero ().setX (model.getHero ().getX () - 4);
				model.getHero ().restartAnimation (Moving.RIGHT);
			}
		}
		else {
			if (!model.getHero ().isWalking ()) model.getHero ().setWalking (true);
			if (Collision.collisionMapRightEdge (model.getHero ())) {
				if (this.isNextMap (1, 0)) this.getModel ().changeMap (this.nextMapKey (1, 0), Moving.RIGHT);
				else {
					model.getHero ().setX (model.getHero ().getX () - 4);
				}
			}
			else {
				if (model.getHero ().getCurrentDirection () != Moving.RIGHT.getDirection ()) model.getHero ().restartAnimation (Moving.RIGHT);
				if (Collision.collisionSlowlyTiles (model.getHero (), model.getMap ().getSlowlier ())) {
					model.getHero ().setX (model.getHero ().getX () - 2);
				}
			}
		}
	}

	/**
	 * D&eacute;placement sur la gauche
	 */
	private void leftMoving () {
		this.setReleasedLeftKey (false);
		model.getHero ().setX (model.getHero ().getX () - 4);
		if (Collision.collisionTiles (model.getHero (), model.getMap ().getBlocked ())) {
			if (!this.onEvent ()) {
				if (model.getHero ().isWalking ()) model.getHero ().setWalking (false);
				model.getHero ().setX (model.getHero ().getX () + 4);
				model.getHero ().restartAnimation (Moving.LEFT);
			}
		}
		else {
			if (!model.getHero ().isWalking ()) model.getHero ().setWalking (true);
			if (Collision.collisionMapLeftEdge (model.getHero ())) {
				if (this.isNextMap (-1, 0)) this.getModel ().changeMap (this.nextMapKey (-1, 0), Moving.LEFT);
				else model.getHero ().setX (model.getHero ().getX () + 4);
			}
			else {
				if (model.getHero ().getCurrentDirection () != Moving.LEFT.getDirection ()) model.getHero ().restartAnimation (Moving.LEFT);
				if (Collision.collisionSlowlyTiles (model.getHero (), model.getMap ().getSlowlier ())) model.getHero ().setX (model.getHero ().getX () + 2);
			}
		}
	}

	/**
	 * Gestion des touches relach&eacute;es
	 * 
	 * @param input
	 *            Le clavier en entr&eacute;
	 */
	private void releasedKey (Input input) {
		if (!input.isKeyDown (Input.KEY_UP)) this.setReleasedUpKey (true);
		if (!input.isKeyDown (Input.KEY_DOWN)) this.setReleasedDownKey (true);
		if (!input.isKeyDown (Input.KEY_LEFT)) this.setReleasedLeftKey (true);
		if (!input.isKeyDown (Input.KEY_RIGHT)) this.setReleasedRightKey (true);
	}

	/**
	 * Test s'il existe une map suivante
	 * 
	 * @param x
	 *            l'abscisse de la map
	 * @param y
	 *            L'ordonn&eacute;e de la map
	 * @return Vrai s'il existe une map suivante (Faux sinon)
	 */
	private boolean isNextMap (int x, int y) {
		if (model.getListMaps ().containsKey (this.nextMapKey (x, y))) return true;
		return false;
	}

	/**
	 * Calcul de la valeur de la cl&eacute; de la map suivante
	 * 
	 * @param x
	 *            l'abscisse de la map
	 * @param y
	 *            L'ordonn&eacute;e de la map
	 * @return La valeur de la cl&eacute; de la map potentiellement suivante
	 */
	private String nextMapKey (int x, int y) {
		String keyCurrentMap = this.getModel ().getMap ().getKey (); // On récupère la clé courante de la map
		String keyNextMap = (Integer.valueOf (keyCurrentMap.split ("-")[0]) + x) + "-" + (Integer.valueOf (keyCurrentMap.split ("-")[1]) + y); // On crée la clé de la carte suivante
		return keyNextMap;
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param model
	 *            the model to set
	 */
	public void setModel (Model model) {
		this.model = model;
	}

	/**
	 * @return the model
	 */
	public Model getModel () {
		return model;
	}

	/**
	 * @return the releasedDownKey
	 */
	public boolean isReleasedDownKey () {
		return releasedDownKey;
	}

	/**
	 * @param releasedDownKey
	 *            the releasedDownKey to set
	 */
	public void setReleasedDownKey (boolean releasedDownKey) {
		this.releasedDownKey = releasedDownKey;
	}

	/**
	 * @return the releasedLeftKey
	 */
	public boolean isReleasedLeftKey () {
		return releasedLeftKey;
	}

	/**
	 * @param releasedLeftKey
	 *            the releasedLeftKey to set
	 */
	public void setReleasedLeftKey (boolean releasedLeftKey) {
		this.releasedLeftKey = releasedLeftKey;
	}

	/**
	 * @return the releasedRightKey
	 */
	public boolean isReleasedRightKey () {
		return releasedRightKey;
	}

	/**
	 * @param releasedRightKey
	 *            the releasedRightKey to set
	 */
	public void setReleasedRightKey (boolean releasedRightKey) {
		this.releasedRightKey = releasedRightKey;
	}

	/**
	 * @return the releasedUpKey
	 */
	public boolean isReleasedUpKey () {
		return releasedUpKey;
	}

	/**
	 * @param releasedUpKey
	 *            the releasedUpKey to set
	 */
	public void setReleasedUpKey (boolean releasedUpKey) {
		this.releasedUpKey = releasedUpKey;
	}
}
