// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * 
 */
package controller;

import java.awt.Point;
import java.util.HashMap;

import model.characters.Hero;

/**
 * <p>
 * <code>EventCollision</code>Permet le calcul des collisions avec les
 * évènements lors de l'appuie de la touche Action.
 * </p>
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-20, 8:29pm<br>
 *          0.1.2 : 2011-01-27, 5:00pm
 */
public class EventCollision {
	////////////////////////
	// Méthodes statiques //
	////////////////////////
	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionEvent (Hero box, HashMap<String, Point> event) {
		String eventStr = "";

		switch (box.getMoving ()) {
			case UP:
				return collisionUpEvent (box, event);
			case DOWN:
				return collisionDownEvent (box, event);
			case RIGHT:
				return collisionRightEvent (box, event);
			case LEFT:
				return collisionLeftEvent (box, event);
		}
		return eventStr;
	}
	
	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionEventOn (Hero box, HashMap<String, Point> event) {
		int i1 = box.getX () / 32;
		int j1 = box.getY () / 32;
		return tileIsEvent (i1, j1, event);
	}

	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionUpEvent (Hero box, HashMap<String, Point> event) {
		String eventStr = "";

		int i1 = box.getX () / 32;
		int j1 = (box.getY () / 32) - 1;
		int i2 = (box.getX () + 24 - 1) / 32;
		int i;
		for (i = i1; i <= i2; i++) {
			String str;
			if (!(str = tileIsEvent (i, j1, event)).equals ("")) return str;
		}

		return eventStr;
	}
	
	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionDownEvent (Hero box, HashMap<String, Point> event) {
		String eventStr = "";

		int i1 = box.getX () / 32;
		int j1 = ((box.getY ()) / 32) + 1;
		int i2 = (box.getX () + 24 - 1) / 32;
		int i;
		for (i = i1; i <= i2; i++) {
			String str;
			if (!(str = tileIsEvent (i, j1, event)).equals ("")) return str;
		}

		return eventStr;
	}
	
	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionRightEvent (Hero box, HashMap<String, Point> event) {
		String eventStr = "";

		int i1 = box.getY () / 32;
		int j1 = (box.getX () / 32) + 1;
		int i2 = (box.getY () + 16 - 1) / 32;
		int i;
		for (i = i1; i <= i2; i++) {
			String str;
			if (!(str = tileIsEvent (j1, i, event)).equals ("")) return str;
		}

		return eventStr;
	}
	
	/**
	 * @param box
	 * @param event
	 * @return
	 */
	public static String collisionLeftEvent (Hero box, HashMap<String, Point> event) {
		String eventStr = "";

		int i1 = box.getY () / 32;
		int j1 = (box.getX () / 32) - 1;
		int i2 = (box.getY () + 16 - 1) / 32;
		int i;
		for (i = i1; i <= i2; i++) {
			String str;
			if (!(str = tileIsEvent (j1, i, event)).equals ("")) return str;
		}

		return eventStr;
	}

	/**
	 * @param i
	 * @param j
	 * @param event
	 * @return
	 */
	private static String tileIsEvent (int i, int j, HashMap<String, Point> event) {
		if (event.containsValue (new Point (i, j))) return getKey (event, new Point (i, j));
		else return "";
	}

	/**
	 * Récupère la clé à partir d'une valeur
	 * 
	 * @param h
	 *            La HashMap
	 * @param p
	 *            La valeur
	 * @return La clé
	 */
	public static String getKey (HashMap<String, Point> h, Point p) {
		for (String str : h.keySet ())
			if (h.get (str).equals (p)) return str;
		return null;
	}
}
