// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * Regroupera tous les états du jeu
 */
package gameStates;

import model.Model;
import observer.Observer;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

import controller.Controller;

/**
 * Etat du jeu
 * 
 * @author Jérémy Chevrier
 * @version 0.1.1 : 2011-01-19, 3:53pm<br>
 *          0.1.2 : 2011-01-19, 8:35pm<br>
 */
public class Game extends BasicGameState implements Observer {

	///////////////
	// Attributs //
	///////////////
	/**
	 * L'état
	 */
	private int state;
	/**if (!"false".equals (eventValue)) {
					String globalKey = map.getTileProperty (tileID, "globalKey", "false");
					if (!"false".equals (globalKey)) {
						eventPoint.put (globalKey, new Point (x, y));
						EventXMLLoader.getHashMapEvent ().get (globalKey).setLocalkey (Integer.valueOf (map.getTileProperty (tileID, "localKey", "false")));
						if (eventValue.equalsIgnoreCase ("npcS")) {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (this.centerXElement (x, NpcsStaticLoader.getHashMapImage ().get (globalKey)));
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (this.centerYElement (y, NpcsStaticLoader.getHashMapImage ().get (globalKey)));
						}
						else if (eventValue.equalsIgnoreCase ("npcD")) {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (x * 32);
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (y * 32);
							listNPCMaps.add (new AnimationNPCMap (globalKey, x * 32, y * 32));
						}
						else {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (x);
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (y);
						}
					}
				}
	 * Le controleur
	 */
	private Controller controller;
	
	/**
	 * Constructeur
	 * 
	 * @param state
	 *            Correspond à l'état du jeu
	 */
	public Game (int state) {
		this.setState (state);
		controller = new Controller (new Model ());
	}

	////////////////////////
	// Méthode redéfinies //
	////////////////////////
	@Override
	public int getID () {
		return state;
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#init(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame)
	 */
	@Override
	public void init (GameContainer arg0, StateBasedGame arg1) throws SlickException {
		this.initialisize(false, "");
	}

	@Override
	public void render (GameContainer arg0, StateBasedGame arg1, Graphics arg2) throws SlickException {
		controller.getModel ().getMap ().drawLowLayerMap (); // On dessine la map au-dessous du personnage
		controller.getModel ().getMap ().drawNpc (); // On dessine le bas des Npcs
		controller.getModel ().getMap ().drawAnimation ();
		controller.getModel ().getHero ().drawAnimation (); // On dessine le personnage
		controller.getModel ().getMap ().drawNpcTop (); // On dessine le haut des Npcs
		controller.getModel ().getMap ().drawHighLayerMap (); // On dessine la map au-dessus du personnage
		if (controller.getModel ().isOutMap ()) {
			controller.getModel ().getMenu ().drawMenu ();
		}
	}

	@Override
	public void update (GameContainer gc, StateBasedGame sbg, int delta) throws SlickException {
		if (this.getController () != null) {
			
			controller.getModel ().getMap ().decAnimation (delta); // On décrémente les compteurs des animations
			
			Input input = gc.getInput ();
			controller.controlInput (input, sbg);
			
			controller.fight ();
		}
	}

	/**
	 * @return the controller
	 */
	public Controller getController () {
		return controller;
	}

	/**
	 * @param controller the controller to set
	 */
	public void setController (Controller controller) {
		this.controller = controller;
	}

	@Override
	public void initialisize (boolean needingLoad, String name) {
		if (needingLoad) this.getController ().getModel ().loadModel (needingLoad, name);
		else this.getController ().getModel ().loadModel (needingLoad, name);		
	}

	/**
	 * @return the state
	 */
	public int getState () {
		return state;
	}

	/**
	 * @param state the state to set
	 */
	public void setState (int state) {
		this.state = state;
	}
}
