// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * 
 */
package gameStates;

import java.io.File;
import java.util.ArrayList;

import loader.FontLoader;
import loader.menus.MenuLoader;
import model.various.TxtFileFilter;
import observer.Observable;
import observer.Observer;

import org.newdawn.slick.Color;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

/**
 * @author jeremy
 * 
 */
public class Loading extends BasicGameState implements Observable {

	/**
	 * Le state
	 */
	private int state;
	/**
	 * Observer
	 */
	private ArrayList<Observer> listObserver = new ArrayList<Observer> ();
	/**
	 * Le curseur
	 */
	private int cursor;
	/**
	 * 
	 */
	private String[] fileSave;

	public Loading (int state) {
		this.setState (state);
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#init(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame)
	 */
	@Override
	public void init (GameContainer gc, StateBasedGame sbg) throws SlickException {
		File directory = new File ("data/saves/");
		File[] listFile = directory.listFiles (new TxtFileFilter ());
		if (listFile.length > 2) fileSave = new String[3];
		else fileSave = new String[listFile.length];
		for (int i = 0; i < fileSave.length; i++)
			fileSave[i] = listFile[i].getName ();
		this.setCursor (0);
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#render(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, org.newdawn.slick.Graphics)
	 */
	@Override
	public void render (GameContainer gc, StateBasedGame sbg, Graphics g) throws SlickException {
		MenuLoader.getHashMapImage ().get ("20").draw ();
		for (int i = 0; i < 3; i++) {
			if (i >= fileSave.length) MenuLoader.getHashMapImage ().get ("23").draw (64, 96 + (i * 128));
			else MenuLoader.getHashMapImage ().get ("21").draw (64, 96 + (i * 128));
		}
		MenuLoader.getHashMapImage ().get ("22").draw (68, 101 + (cursor * 128));
		for (int i = 0; i < fileSave.length; i++) {
			//g.drawString (fileSave[i].split ("_")[0], 96, 128 + (i * 128));
			FontLoader.getFontShadow ().drawString (97, 129 + (i * 128), fileSave[i].split ("_")[0]);
			FontLoader.getFont ().drawString (96, 128 + (i * 128), fileSave[i].split ("_")[0]);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#update(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, int)
	 */
	@Override
	public void update (GameContainer gc, StateBasedGame sbg, int delta) throws SlickException {
		Input input = gc.getInput ();
		if (input.isKeyPressed (Input.KEY_ENTER)) {
			this.updateObserver (true);
			sbg.enterState (1, new FadeOutTransition (Color.black, 2000), new FadeInTransition (Color.black, 2000));
		}
		else if (input.isKeyPressed (Input.KEY_UP)) {
			if (this.getCursor () > 0) this.setCursor (this.getCursor () - 1);
		}
		else if (input.isKeyPressed (Input.KEY_DOWN)) {
			if (this.getCursor () < this.getFileSave ().length - 1 && this.getCursor () != 2) this.setCursor (this.getCursor () + 1);
		}
		else if (input.isKeyPressed (Input.KEY_ESCAPE)) {
			sbg.enterState (0);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.newdawn.slick.state.BasicGameState#getID()
	 */
	@Override
	public int getID () {
		return 4;
	}

	/////////////////////////
	// Partie Observateurs //
	/////////////////////////
	@Override
	public void addObserver (Observer observer) {
		listObserver.add (observer);
	}

	@Override
	public void updateObserver (boolean needingLoading) {
		for (Observer observer : listObserver) {
			if (needingLoading) observer.initialisize (true, fileSave[this.getCursor ()]);
			else observer.initialisize (false, "");
		}
	}

	@Override
	public void delObserver () {
		listObserver = new ArrayList<Observer> ();
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param state
	 *            the state to set
	 */
	public void setState (int state) {
		this.state = state;
	}

	/**
	 * @return the state
	 */
	public int getState () {
		return state;
	}

	/**
	 * @param listObserver
	 *            the listObserver to set
	 */
	public void setListObserver (ArrayList<Observer> listObserver) {
		this.listObserver = listObserver;
	}

	/**
	 * @return the listObserver
	 */
	public ArrayList<Observer> getListObserver () {
		return listObserver;
	}

	/**
	 * @param cursor
	 *            the cursor to set
	 */
	public void setCursor (int cursor) {
		this.cursor = cursor;
	}

	/**
	 * @return the cursor
	 */
	public int getCursor () {
		return cursor;
	}

	/**
	 * @return the fileSave
	 */
	public String[] getFileSave () {
		return fileSave;
	}

	/**
	 * @param fileSave
	 *            the fileSave to set
	 */
	public void setFileSave (String[] fileSave) {
		this.fileSave = fileSave;
	}
}
