// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * 
 */
package gameStates;

import java.io.IOException;
import java.util.ArrayList;

import loader.AnimatedLoader;
import loader.FontLoader;
import loader.MapLoader;
import loader.PreLoader;
import loader.SKillsPropLoader;
import loader.SkillsLoader;
import loader.SoundsLoader;
import loader.characters.MonsterLoader;
import loader.characters.MonstersPropLoader;
import loader.characters.NpcsDynamicLoader;
import loader.characters.NpcsStaticLoader;
import loader.characters.SpiritLoader;
import loader.events.EventXMLLoader;
import loader.items.ItemsLoader;
import loader.menus.AvatarLoader;
import loader.menus.DiversLoader;
import loader.menus.MenuLoader;
import observer.Observable;
import observer.Observer;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.loading.DeferredResource;
import org.newdawn.slick.loading.LoadingList;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

/**
 * @author jeremy
 * 
 */
public class MainMenu extends BasicGameState implements Observable {

	/**
	 * numéro de l'état
	 */
	private int state;
	/**
	 * Observer
	 */
	private ArrayList<Observer> listObserver = new ArrayList<Observer> ();
	// Barre de chargement
	/**
	 * 
	 */
	private String lastLoaded;
	/**
	 * Pour les ressources suivantes
	 */
	private DeferredResource nextResource; // Ressource suivante à charger
	/**
	 * Savoir si le chargement est terminé
	 */
	private boolean started = false;
	/**
	 * Savoir si on est sur Nouveau
	 */
	private boolean newGame;
	/**
	 * Savoir si on est sur Charger
	 */
	private boolean loadGame;
	/**
	 * Savoir si on est sur Quitter
	 */
	private boolean quitGame;
	/**
	 * X du Background
	 */
	private int xBackground;
	/**
	 * Y du Background
	 */
	private int yBackGround;
	/**
	 * L'orientation des abscisses
	 */
	private boolean xOrientation;
	/**
	 * L'orientation des ordonnées
	 */
	private boolean yOrientation;
	/**
	 * Le compteur de delta
	 */
	private int deltaInc;

	public MainMenu (int state) {
		this.setState (state);
	}

	/*
	 * (non-Javadoc)"data/ressources/characters/spirit/images/"
	 * @see
	 * org.newdawn.slick.state.GameState#init(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame)
	 */
	@Override
	public void init (GameContainer gc, StateBasedGame sbg) throws SlickException {
		this.setxBackground (-30);
		this.setyBackGround (-100);
		this.setDeltaInc (0);
		this.setxOrientation (true);
		PreLoader.loadPreLoading ();
		LoadingList.setDeferredLoading (true);
		EventXMLLoader.eventsLoader ("data/ressources/events/properties/");
		AnimatedLoader.spritesLoader ("data/ressources/other/maps/images/");
		MapLoader.mapLoader ();
		SkillsLoader.imagesLoader ("data/ressources/battle/skills/images/");
		SpiritLoader.imagesLoader ("data/ressources/characters/spirit/images/");
		FontLoader.loadFonts ();
		AvatarLoader.imagesLoader ("");
		NpcsStaticLoader.imagesLoader ("data/ressources/characters/npcs/images/staticNPC/");
		NpcsDynamicLoader.spriteLoader ("data/ressources/characters/npcs/images/dynamicNPC/");
		MonsterLoader.imagesLoader ("data/ressources/characters/monsters/images/");
		ItemsLoader.imagesLoader ("data/ressources/items/other/images/");
		MenuLoader.imagesLoader ("data/ressources/menus/images/");
		new DiversLoader (); // statique aussi ^^
		System.out.println ("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
		SoundsLoader.chargeSounds ("data/ressources/sounds/music/"); // Ne fonctionne pas sur mon système !!! Slick (à re-install)
		System.out.println ("~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
		SKillsPropLoader.skillLoader ("");
		MonstersPropLoader.monsterLoader ("");
		this.setNewGame (true);
		this.setLoadGame (false);
		this.setQuitGame (false);
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#render(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, org.newdawn.slick.Graphics)
	 */
	@Override
	public void render (GameContainer gc, StateBasedGame sbg, Graphics g) throws SlickException {
		if (!this.isStarted ()) {
			PreLoader.getBackground ().draw ();
			if (nextResource != null) {
//				PreLoader..drawString ("Loading: " + lastLoaded, 10, 100);
			}

			int total = LoadingList.get ().getTotalResources ();
			int loaded = LoadingList.get ().getTotalResources () - LoadingList.get ().getRemainingResources ();
			
			PreLoader.getBar ().draw (72, 448, (639 * loaded / total), 26);
//			float bar = loaded / (float) total;
//			g.fillRect (100, 150, loaded * 12, 20);
//			g.drawRect (100, 150, total * 12, 20);
//			g.drawString ("" + ((int) (bar * 100)) + "%", total * 12 + 110, 150);
		}
		else {
			MenuLoader.getHashMapImage ().get ("11").draw (this.getxBackground (), this.getyBackGround ());
			MenuLoader.getHashMapImage ().get ("03").draw ();
			if (this.isNewGame ()) MenuLoader.getHashMapImage ().get ("07").draw (316, 332);
			else MenuLoader.getHashMapImage ().get ("06").draw (316, 332);
			if (this.isLoadGame ()) MenuLoader.getHashMapImage ().get ("05").draw (316, 364);
			else MenuLoader.getHashMapImage ().get ("04").draw (316, 364);
			if (this.isQuitGame ()) MenuLoader.getHashMapImage ().get ("09").draw (316, 396);
			else MenuLoader.getHashMapImage ().get ("08").draw (316, 396);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#update(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, int)
	 */
	@Override
	public void update (GameContainer gc, StateBasedGame sbg, int delta) throws SlickException {
		if (nextResource != null) {
			try {
				nextResource.load ();
//				try {
//					Thread.sleep (50);
//				}
//				catch (Exception e) {}
			}
			catch (IOException e) {
				throw new SlickException ("Failed to load: " + nextResource.getDescription (), e);
			}
			nextResource = null;
		}

		if (LoadingList.get ().getRemainingResources () > 0) {
			nextResource = LoadingList.get ().getNext ();
			setLastLoaded (nextResource.getDescription ());
		}
		else {
			deltaInc += delta;
			if (deltaInc > 30) {
				this.moveBackground ();
				deltaInc = 0;
			}
			setStarted (true);
			Input input = gc.getInput ();
			if (input.isKeyPressed (Input.KEY_UP)) {
				if (!this.isNewGame ()) {
					if (this.isLoadGame ()) {
						this.setLoadGame (false);
						this.setNewGame (true);
					}
					else if (this.isQuitGame ()) {
						this.setQuitGame (false);
						this.setLoadGame (true);
					}
				}
			}
			else if (input.isKeyPressed (Input.KEY_DOWN)) {
				if (!this.isQuitGame ()) {
					if (this.isNewGame ()) {
						this.setNewGame (false);
						this.setLoadGame (true);
					}
					else if (this.isLoadGame ()) {
						this.setLoadGame (false);
						this.setQuitGame (true);
					}
				}
			}
			else if (input.isKeyPressed (Input.KEY_ENTER)) {
				if (this.isNewGame ()) {
					this.updateObserver (false);
					sbg.enterState (3);
				}
				else if (this.isLoadGame ()) {
					sbg.enterState (4);
				}
				else if (this.isQuitGame ()) {
					System.exit (0);
				}
			}
		}
	}

	private void moveBackground () {
		// X
		if (this.getxBackground () < (-108)) {
			this.setxOrientation (true);
			if (this.isxOrientation ()) this.setxBackground (this.getxBackground () + 1);
			else this.setxBackground (this.getxBackground () - 1);
		}
		else if (this.getxBackground () < 0) {
			if (this.isxOrientation ()) this.setxBackground (this.getxBackground () + 1);
			else this.setxBackground (this.getxBackground () - 1);
		}
		else {
			this.setxOrientation (false);
			if (this.isxOrientation ()) this.setxBackground (this.getxBackground () + 1);
			else this.setxBackground (this.getxBackground () - 1);
		}
		// Y
		if (this.getyBackGround () < (-332)) {
			this.setyOrientation (true);
			if (this.isyOrientation ()) this.setyBackGround (this.getyBackGround () + 1);
			else this.setyBackGround (this.getyBackGround () - 1);
		}
		else if (this.getxBackground () < 0) {
			if (this.isyOrientation ()) this.setyBackGround (this.getyBackGround () + 1);
			else this.setyBackGround (this.getyBackGround () - 1);
		}
		else {
			this.setyOrientation (false);
			if (this.isyOrientation ()) this.setyBackGround (this.getyBackGround () + 1);
			else this.setyBackGround (this.getyBackGround () - 1);
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.newdawn.slick.state.BasicGameState#getID()
	 */
	@Override
	public int getID () {
		return 0;
	}

	/////////////////////////
	// Partie Observateurs //
	/////////////////////////
	@Override
	public void addObserver (Observer observer) {
		listObserver.add (observer);
	}

	@Override
	public void updateObserver (boolean needingLoading) {
		for (Observer observer : listObserver) {
			if (needingLoading) observer.initialisize (true, "");
			else observer.initialisize (false, "");
		}
	}

	@Override
	public void delObserver () {
		listObserver = new ArrayList<Observer> ();
	}

	/**
	 * @param state
	 *            the state to set
	 */
	public void setState (int state) {
		this.state = state;
	}

	/**
	 * @return the state
	 */
	public int getState () {
		return state;
	}

	/**
	 * @param nextResource
	 *            the nextResource to set
	 */
	public void setNextResource (DeferredResource nextResource) {
		this.nextResource = nextResource;
	}

	/**
	 * @return the nextResource
	 */
	public DeferredResource getNextResource () {
		return nextResource;
	}

	/**
	 * @param started
	 *            the started to set
	 */
	public void setStarted (boolean started) {
		this.started = started;
	}

	/**
	 * @return the started
	 */
	public boolean isStarted () {
		return started;
	}

	/**
	 * @param newGame
	 *            the newGame to set
	 */
	public void setNewGame (boolean newGame) {
		this.newGame = newGame;
	}

	/**
	 * @return the newGame
	 */
	public boolean isNewGame () {
		return newGame;
	}

	/**
	 * @param loadGame
	 *            the loadGame to set
	 */
	public void setLoadGame (boolean loadGame) {
		this.loadGame = loadGame;
	}

	/**
	 * @return the loadGame
	 */
	public boolean isLoadGame () {
		return loadGame;
	}

	/**
	 * @param quitGame
	 *            the quitGame to set
	 */
	public void setQuitGame (boolean quitGame) {
		this.quitGame = quitGame;
	}

	/**
	 * @return the quitGame
	 */
	public boolean isQuitGame () {
		return quitGame;
	}

	/**
	 * @param xBackground
	 *            the xBackground to set
	 */
	public void setxBackground (int xBackground) {
		this.xBackground = xBackground;
	}

	/**
	 * @return the xBackground
	 */
	public int getxBackground () {
		return xBackground;
	}

	/**
	 * @param yBackGround
	 *            the yBackGround to set
	 */
	public void setyBackGround (int yBackGround) {
		this.yBackGround = yBackGround;
	}

	/**
	 * @return the yBackGround
	 */
	public int getyBackGround () {
		return yBackGround;
	}

	/**
	 * @param deltaInc
	 *            the deltaInc to set
	 */
	public void setDeltaInc (int deltaInc) {
		this.deltaInc = deltaInc;
	}

	/**
	 * @return the deltaInc
	 */
	public int getDeltaInc () {
		return deltaInc;
	}

	/**
	 * @param xOrientation
	 *            the xOrientation to set
	 */
	public void setxOrientation (boolean xOrientation) {
		this.xOrientation = xOrientation;
	}

	/**
	 * @return the xOrientation
	 */
	public boolean isxOrientation () {
		return xOrientation;
	}

	/**
	 * @param yOrientation
	 *            the yOrientation to set
	 */
	public void setyOrientation (boolean yOrientation) {
		this.yOrientation = yOrientation;
	}

	/**
	 * @return the yOrientation
	 */
	public boolean isyOrientation () {
		return yOrientation;
	}

	/**
	 * @return the listObserver
	 */
	public ArrayList<Observer> getListObserver () {
		return listObserver;
	}

	/**
	 * @param listObserver the listObserver to set
	 */
	public void setListObserver (ArrayList<Observer> listObserver) {
		this.listObserver = listObserver;
	}

	/**
	 * @param lastLoaded the lastLoaded to set
	 */
	public void setLastLoaded (String lastLoaded) {
		this.lastLoaded = lastLoaded;
	}

	/**
	 * @return the lastLoaded
	 */
	public String getLastLoaded () {
		return lastLoaded;
	}
}
