// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * 
 */
package gameStates;

import loader.FontLoader;
import loader.characters.HeroLoader;
import loader.characters.SpiritLoader;
import loader.menus.MenuLoader;

import org.newdawn.slick.Animation;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.HorizontalSplitTransition;

import controller.Controller;

/**
 * @author Jeremy Chevrier
 * 
 */
public class MenuHero extends BasicGameState {

	private int state = 0;
	private boolean quit;
	private Controller controller;
	private Animation animationH, animationS1;

	public MenuHero (int state, Controller controller) {
		this.setState (state);
		this.setController (controller);
	}

	@Override
	public void init (GameContainer gc, StateBasedGame sbg) throws SlickException {
		setQuit (false);
		this.loadAnimations ();
	}

	@Override
	public void render (GameContainer gc, StateBasedGame sbg, Graphics g) throws SlickException {
		MenuLoader.getHashMapImage ().get ("10").draw ();
		if (this.getController () != null) {
			FontLoader.getFontShadow ().drawString (11, 471, "Argent : " + this.getController ().getModel ().getHero ().getMoney ());
			FontLoader.getFont ().drawString (10, 470, "Argent : " + this.getController ().getModel ().getHero ().getMoney ());
			FontLoader.getFontShadow ().drawString (11, 518, "Quitter");
			FontLoader.getFont ().drawString (10, 517, "Quitter");
			if (this.isQuit ()) MenuLoader.getHashMapImage ().get ("12").draw (10, 540);
			FontLoader.getFontShadow ().drawString (320, 365, this.getController ().getModel ().getHero ().getListSpirit ().get (0).getName ().split ("_")[0]);
			FontLoader.getFont ().drawString (319, 364, this.getController ().getModel ().getHero ().getListSpirit ().get (0).getName ().split ("_")[0]);
			animationH.draw (300, 100);
			FontLoader.getFontShadow ().drawString (320, 65, this.getController ().getModel ().getHero ().getName ().split ("_")[0]);
			FontLoader.getFont ().drawString (319, 64, this.getController ().getModel ().getHero ().getName ().split ("_")[0]);
			animationS1.draw (300, 350);
			FontLoader.getFontShadow ().drawString (501, 151, this.getController ().getModel ().getHero ().getHp () + " / " + this.getController ().getModel ().getHero ().getMaxHP () + " PV");
			FontLoader.getFont ().drawString (500, 150, this.getController ().getModel ().getHero ().getHp () + " / " + this.getController ().getModel ().getHero ().getMaxHP () + " PV");
			FontLoader.getFontShadow ().drawString (501, 451, this.getController ().getModel ().getHero ().getListSpirit ().get (0).getHp () + " / " + this.getController ().getModel ().getHero ().getListSpirit ().get (0).getMaxHP () + " PV");
			FontLoader.getFont ().drawString (500, 450, this.getController ().getModel ().getHero ().getListSpirit ().get (0).getHp () + " / " + this.getController ().getModel ().getHero ().getListSpirit ().get (0).getMaxHP () + " PV");
		}
	}

	@Override
	public void update (GameContainer gc, StateBasedGame sbg, int delta) throws SlickException {
		Input input = gc.getInput ();
		if (input.isKeyPressed (Input.KEY_ESCAPE)) sbg.enterState (1, null, new HorizontalSplitTransition ());
		else if (input.isKeyPressed (Input.KEY_Q)) sbg.enterState (0, null, new FadeInTransition ());
		else if (input.isKeyPressed (Input.KEY_DOWN)) this.setQuit (true);
		else if (input.isKeyPressed (Input.KEY_ENTER) && this.isQuit ()) sbg.enterState (0, null, new FadeInTransition ());
	}
	
	private void loadAnimations () {
		// On charge la liste des animations
		animationH = new Animation (HeroLoader.getSpriteSheetHeroFight (), 0, 0, 1, 0, true, 200, true);
		animationS1 = new Animation (SpiritLoader.getHashMapSprite ().get ("01"), 0, 0, 3, 0, true, 200, true);
		animationH.start ();
		animationS1.start ();
	}

	@Override
	public int getID () {
		return state;
	}

	/**
	 * @param state
	 *            the state to set
	 */
	public void setState (int state) {
		this.state = state;
	}

	/**
	 * @return the state
	 */
	public int getState () {
		return state;
	}

	/**
	 * @param controller
	 *            the controller to set
	 */
	public void setController (Controller controller) {
		this.controller = controller;
	}

	/**
	 * @return the controller
	 */
	public Controller getController () {
		return controller;
	}

	/**
	 * @param quit
	 *            the quit to set
	 */
	public void setQuit (boolean quit) {
		this.quit = quit;
	}

	/**
	 * @return the quit
	 */
	public boolean isQuit () {
		return quit;
	}

	/**
	 * @param animationS1 the animationS1 to set
	 */
	public void setAnimationS1 (Animation animationS1) {
		this.animationS1 = animationS1;
	}

	/**
	 * @return the animationS1
	 */
	public Animation getAnimationS1 () {
		return animationS1;
	}

}
