// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * 
 */
package gameStates;

import java.util.ArrayList;

import loader.menus.MenuLoader;

import observer.Observable;
import observer.Observer;

import org.newdawn.slick.GameContainer;
import org.newdawn.slick.Graphics;
import org.newdawn.slick.Input;
import org.newdawn.slick.KeyListener;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.BasicGameState;
import org.newdawn.slick.state.StateBasedGame;

/**
 * @author jeremy
 * 
 */
public class Name extends BasicGameState implements Observable, Observer, KeyListener {

	/**
	 * Le state
	 */
	private int state;
	/**
	 * Observer
	 */
	private ArrayList<Observer> listObserver = new ArrayList<Observer> ();
	/**
	 * Le nom du perso
	 */
	private String name;
	/**
	 * 
	 */
	private boolean endState;
	/**
	 * Le compteur pour pallier au prob du doublage des lettres
	 */
	private int deltaInc;

	public Name (int state) {
		this.setState (state);
		this.setName ("");
		this.setEndState (false);
		this.setDeltaInc (200);
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#init(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame)
	 */
	@Override
	public void init (GameContainer arg0, StateBasedGame arg1) throws SlickException {
		// Nothing
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#render(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, org.newdawn.slick.Graphics)
	 */
	@Override
	public void render (GameContainer gc, StateBasedGame sbg, Graphics g) throws SlickException {
		MenuLoader.getHashMapImage ().get ("19").draw ();
		g.drawString (getName (), 340, 300);
		g.drawString (getName (), 340, 300);
	}

	/*
	 * (non-Javadoc)
	 * @see
	 * org.newdawn.slick.state.GameState#update(org.newdawn.slick.GameContainer,
	 * org.newdawn.slick.state.StateBasedGame, int)
	 */
	@Override
	public void update (GameContainer gc, StateBasedGame sbg, int delta) throws SlickException {
		this.setDeltaInc (this.getDeltaInc () - delta);
		if (this.getDeltaInc () < 0) this.setDeltaInc (150);
		Input input = gc.getInput ();
		input.addKeyListener (this);
		if (input.isKeyPressed (Input.KEY_ENTER)) {
			this.updateObserver (false);
			input.removeAllKeyListeners ();
			this.setEndState (true);
			sbg.enterState (5);
		} else if (input.isKeyPressed (Input.KEY_BACK)) {
			if (this.getName ().length () > 0) {
				String nameTmp = "";
				for (int i=0; i<this.getName ().length () - 1; i++) 
					nameTmp +=this.getName ().charAt (i);
				this.setName (nameTmp);
			}
		}
	}

	/*
	 * (non-Javadoc)
	 * @see org.newdawn.slick.state.BasicGameState#getID()
	 */
	@Override
	public int getID () {
		return 3;
	}

	/////////////////////////
	// Partie Observateurs //
	/////////////////////////
	@Override
	public void addObserver (Observer observer) {
		listObserver.add (observer);
	}

	@Override
	public void updateObserver (boolean needingLoading) {
		for (Observer observer : listObserver) {
			if (needingLoading) observer.initialisize (true, "");
			else observer.initialisize (false, this.getName ());
		}
	}

	@Override
	public void delObserver () {
		listObserver = new ArrayList<Observer> ();
	}

	// redéfintion
	@Override
	public void keyPressed (int key, char c) {
		if (!this.isEndState ()) {
			if (this.getDeltaInc () < 150) {
				String nameTmp = this.getName ();
				nameTmp += c;
				if (nameTmp.matches ("[a-zA-Z]*") && nameTmp.length () < 9) this.setName (nameTmp);
				this.setDeltaInc (200);
			}
		}
	}

	@Override
	public void keyReleased (int key, char c) {}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param state
	 *            the state to set
	 */
	public void setState (int state) {
		this.state = state;
	}

	/**
	 * @return the state
	 */
	public int getState () {
		return state;
	}

	/**
	 * @param listObserver
	 *            the listObserver to set
	 */
	public void setListObserver (ArrayList<Observer> listObserver) {
		this.listObserver = listObserver;
	}

	/**
	 * @return the listObserver
	 */
	public ArrayList<Observer> getListObserver () {
		return listObserver;
	}

	/**
	 * @param name
	 *            the name to set
	 */
	public void setName (String name) {
		this.name = name;
	}

	/**
	 * @return the name
	 */
	public String getName () {
		return name;
	}

	/**
	 * @param endState
	 *            the endState to set
	 */
	public void setEndState (boolean endState) {
		this.endState = endState;
	}

	/**
	 * @return the endState
	 */
	public boolean isEndState () {
		return endState;
	}

	/**
	 * @param deltaInc
	 *            the deltaInc to set
	 */
	public void setDeltaInc (int deltaInc) {
		this.deltaInc = deltaInc;
	}

	/**
	 * @return the deltaInc
	 */
	public int getDeltaInc () {
		return deltaInc;
	}

	@Override
	public void initialisize (boolean needingLoad, String name) {
		this.setName ("");
		this.setEndState (false);
	}
}
