// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * Package du lanceur du jeu
 */
package launcher;

import gameStates.Game;
import gameStates.GameOver;
import gameStates.IntroductionMovie;
import gameStates.Loading;
import gameStates.MainMenu;
import gameStates.MenuHero;
import gameStates.Name;

import org.newdawn.slick.AppGameContainer;
import org.newdawn.slick.GameContainer;
import org.newdawn.slick.SlickException;
import org.newdawn.slick.state.StateBasedGame;

/**
 * Permet de lancer le jeu
 * 
 * @author Jérémy Chevrier
 * @version 0.1.1: 2011-01-19, 8:36pm<br>
 */
public class GameLauncher extends StateBasedGame {

	public static final int MAINMENUSTATE = 0;
	public static final int GAMESTATE = 1;
	public static final int MENUHEROSTATE = 2;
	public static final int NAMESTATE = 3;
	public static final int LOADINGSTATE = 4;
	public static final int INTRODUCTIONMOVIESTATE = 5;
	public static final int GAMEOVER = 6;

	/**
	 * Le constructeur
	 * 
	 * @param name
	 *            Lance le jeu avec un nom
	 */
	public GameLauncher (String name) {
		super (name);

		// Création des states
		MainMenu mainMenu = new MainMenu (MAINMENUSTATE);
		Name nameHero = new Name (NAMESTATE);
		Game game = new Game (GAMESTATE);
		MenuHero menuHero = new MenuHero (MENUHEROSTATE, game.getController ());
		Loading loading = new Loading (LOADINGSTATE);
		IntroductionMovie introductionMovue= new IntroductionMovie (INTRODUCTIONMOVIESTATE);
		GameOver gameOver = new GameOver (GAMEOVER);
		
		// Ajout de l'observer
		mainMenu.addObserver (nameHero);
		nameHero.addObserver (game);
		loading.addObserver (game);
		// Ajout des States
		this.addState (mainMenu);
		this.addState (game);
		this.addState (nameHero);
		this.addState (menuHero);
		this.addState (introductionMovue);
		this.addState (loading);
		this.addState (gameOver);
	}

	@Override
	public void initStatesList (GameContainer gameContainer) throws SlickException {
		gameContainer.setTargetFrameRate (42); // Fixons le maximum de FPS de l'application
		gameContainer.setVSync (true); // Fixons la synchronisation verticale

		this.getState (MAINMENUSTATE).init (gameContainer, this);
		this.getState (GAMESTATE).init (gameContainer, this);
		this.getState (NAMESTATE).init (gameContainer, this);
		this.getState (MENUHEROSTATE).init (gameContainer, this);
		this.getState (INTRODUCTIONMOVIESTATE).init (gameContainer, this);
		this.getState (LOADINGSTATE).init (gameContainer, this);
		this.getState (GAMEOVER).init (gameContainer, this);
		this.enterState (MAINMENUSTATE);
	}

	/**
	 * @param args
	 */
	public static void main (String[] args) {
		try {
			AppGameContainer app = new AppGameContainer (new GameLauncher ("JZunBeHa - Projet d'app"));
			app.setDisplayMode (800, 576, false);
			app.setTargetFrameRate(60);
			//			app.setDisplayMode (app.getScreenWidth (), app.getScreenHeight (), true);
			app.setShowFPS (false);
			app.start ();
		}
		catch (SlickException e) {
			e.printStackTrace ();
		}
	}

}
