// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * 
 */
package loader;

import java.awt.Color;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.UnicodeFont;
import org.newdawn.slick.font.effects.ColorEffect;

/**
 * @author Jérémy Chevrier
 * @version 0.1.1 2011-01-30, 11:45am
 */
public class FontLoader {

	/////////////////////////
	// Attributs statiques //
	/////////////////////////
	/**
	 * La font
	 */
	private static UnicodeFont font;
	/**
	 * L'ombre pour la font
	 */
	private static UnicodeFont fontShadow;

	////////////////////////
	// Méthodes statiques //
	////////////////////////
	// Charge une fonte précise
	/**
	 * Charge une font
	 * 
	 * @param pathFont
	 *            le chemin de la font
	 * @param taille
	 *            la taille
	 * @param color
	 *            la couleur
	 * @return une font
	 */
	@SuppressWarnings("unchecked")
	private static UnicodeFont loadingFonts (String pathFont, int taille, Color color) {
		UnicodeFont font = null;

		try {
			font = new UnicodeFont (pathFont, taille, false, false); // On créée une instance de la police
			font.addAsciiGlyphs (); // Add Glyphs
			font.addGlyphs (400, 600); // Add Glyphs
			font.getEffects ().add (new ColorEffect (color));
			font.loadGlyphs (); // Load Glyphs
			return font;
		}
		catch (SlickException e) {
			// TODO Auto-generated catch block
			e.printStackTrace ();
		}
		return font;
	}

	// Charge toutes les fonts
	/**
	 * Charge toutes les fonts
	 */
	public static void loadFonts () {
		setFont (loadingFonts ("data/ressources/fonts/AMERSN__.ttf", 20, new Color (221, 221, 221)));
		setFontShadow (loadingFonts ("data/ressources/fonts/AMERSN__.ttf", 20, new Color (51, 51, 51)));
	}

	//////////////////////////
	// Accesseurs statiques //
	//////////////////////////
	/**
	 * @param font
	 *            the font to set
	 */
	public static void setFont (UnicodeFont font) {
		FontLoader.font = font;
	}

	/**
	 * @return the font
	 */
	public static UnicodeFont getFont () {
		return font;
	}

	/**
	 * @param fontShadow
	 *            the fontShadow to set
	 */
	public static void setFontShadow (UnicodeFont fontShadow) {
		FontLoader.fontShadow = fontShadow;
	}

	/**
	 * @return the fontShadow
	 */
	public static UnicodeFont getFontShadow () {
		return fontShadow;
	}
}
