// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * Ce package va regrouper toutes les classes de chargement des ressources de
 * tout le jeu.
 */
package loader;

import java.io.File;
import java.util.HashMap;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.tiled.TiledMap;

/**
 * <p>
 * <code>MapLoader</code> est une classe qui va charger toutes les cartes du
 * jeu. Une carte est au format .tmx. Ce n'est que de l'xml. Toutes les cartes
 * ont des tilesets au format .png.
 * </p>
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-16, 11:21am <br>
 *          0.1.2 : 2011-01-19, 8:44pm
 */
public class MapLoader {

	////////////////////////////////////
	// Les différents objets TiledMap //
	////////////////////////////////////
	/**
	 * Liste de toutes les cartes avec leur clé respective
	 */
	private static HashMap<String, TiledMap> hashMapMap = new HashMap<String, TiledMap> ();
	/**
	 * La liste des chemins de toutes les cartes à charger
	 */
	private static File[] list;

	////////////////////////
	// Méthodes statiques //
	////////////////////////
	/**
	 * S'occupe de charger une carte avec son tileset.
	 * 
	 * @param tmxPath
	 *            Le chemin du fichier .tmx
	 * @param tilesetPath
	 *            Le chemin du tileset
	 * @return la carte chargée
	 */
	private static TiledMap tmxLoader (String tmxPath, String tilesetPath) {
		TiledMap map = null;
		try {
			map = new TiledMap (tmxPath, tilesetPath);
		}
		catch (SlickException e) {
			System.out.println ("File or tileset not found !");
			e.printStackTrace ();
		}
		return map;
	}

	/**
	 * Charge toutes les cartes du jeu.
	 */
	public static void mapLoader () {
		File directory = new File ("data/ressources/maps/properties/"); // On fixe le dossier contenant les maps
		if (directory.isDirectory ()) list = directory.listFiles (); // On charge la liste des fichiers contenu dans ce dossier
		for (File file : list) {

			if (!file.isDirectory ())
			{
				String[] key = file.getPath ().split ("_")[1].split (".tmx"); // On récupère la clé
				getHashTableMap ().put (key[0], tmxLoader (file.getPath (), "data/ressources/maps")); // On charge la map et on la met dans une hashTable
			}

			if (!file.isDirectory ()) { // Si c'est un fichier, alors on le charge
				String[] key = file.getPath ().split ("_")[1].split (".tmx"); // On récupère la clé
				getHashTableMap ().put (key[0], tmxLoader (file.getPath (), "data/ressources/maps")); // On charge la map et on la met dans une hashTable
			}

		}
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param hashMapMap
	 *            the hashMapMap to set
	 */
	public static void setHashTableMap (HashMap<String, TiledMap> hashTableMap) {
		MapLoader.hashMapMap = hashTableMap;
	}

	/**
	 * @return the hashMapMap
	 */
	public static HashMap<String, TiledMap> getHashTableMap () {
		return hashMapMap;
	}

}
