// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * Ce package va regrouper toutes les classes de chargement des ressources de
 * tout le jeu.
 */
package loader.characters;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;

/**
 * <p>
 * <code>HeroLoader</code> va charger les ressources du h&eacute;ro.
 * </p>
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1: 2011-01-19, 8:43pm<br>
 */
public class HeroLoader {

	///////////////////////////////////
	// Les différents objets du héro //
	///////////////////////////////////
	/**
	 * La feuille de sprites du héro
	 */
	private static SpriteSheet spriteSheetBase;
	private static SpriteSheet spriteSheetHeroFight;

	////////////////////////
	// Méthodes statiques //
	////////////////////////
	/**
	 * @param spriteSheetPath
	 * @param tileWidth
	 * @param tileHeight
	 * @return
	 */
	private static SpriteSheet spriteSheetLoader (String spriteSheetPath, int tileWidth, int tileHeight) {
		SpriteSheet spriteSheet = null;
		try {
			spriteSheet = new SpriteSheet (spriteSheetPath, tileWidth, tileHeight);
		}
		catch (SlickException e) {
			System.out.println ("No spritesheet found !");
			e.printStackTrace ();
		}
		return spriteSheet;
	}

	/**
	 * Charge toutes les images du joueur.
	 */
	public static void imagesLoader () {
		setSpriteSheetBase (spriteSheetLoader ("data/ressources/characters/hero/images/heros.png", 24, 44));
		setSpriteSheetHeroFight (spriteSheetLoader ("data/ressources/characters/hero/images/spriteHeroCombatFINAL.png", 128, 128));
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param spriteSheetBase
	 *            the spriteSheetBase to set
	 */
	public static void setSpriteSheetBase (SpriteSheet spriteSheetBase) {
		HeroLoader.spriteSheetBase = spriteSheetBase;
	}

	/**
	 * @return the spriteSheetBase
	 */
	public static SpriteSheet getSpriteSheetBase () {
		return spriteSheetBase;
	}

	/**
	 * @param spriteSheetHeroFight the spriteSheetHeroFight to set
	 */
	public static void setSpriteSheetHeroFight (SpriteSheet spriteSheetHeroFight) {
		HeroLoader.spriteSheetHeroFight = spriteSheetHeroFight;
	}

	/**
	 * @return the spriteSheetHeroFight
	 */
	public static SpriteSheet getSpriteSheetHeroFight () {
		return spriteSheetHeroFight;
	}
}
