// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * Ce package va regrouper toutes les classes de chargement des ressources de
 * tout le jeu.
 */
package loader.characters;

import java.io.File;
import java.util.HashMap;

import org.newdawn.slick.SlickException;
import org.newdawn.slick.SpriteSheet;

import loader.ImagesLoader;

/**
 * <p>
 * <code>MonsterLoader</code> qui va charger tous les monstres
 * </p>
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-20, 1:49pm
 */
public class MonsterLoader extends ImagesLoader {
	private static HashMap<String, SpriteSheet> hashMapSprite = new HashMap<String, SpriteSheet> ();
	 
	/**
	 * Charge toutes les images des monstres du jeu
	 */
	public static void imagesLoader (String folderPath) {
		File directory = new File (folderPath); // On fixe le dossier contenant les maps
		if (directory.isDirectory ()) setList (directory.listFiles ()); // On charge la liste des fichiers contenu dans ce dossier
		for (File file : getList ()) {
			if (!file.isDirectory ()) { // Si c'est un fichier, alors on le charge
				String[] key = file.getPath ().split ("_")[1].split (".png"); // On récupère la clé
				if (!file.getParentFile ().getName ().equalsIgnoreCase ("sprite")) getHashMapImage ().put (key[0], imageLoader (file.getPath ())); // On charge la map et on la met dans une hashTable
				else if (file.getParentFile ().getName ().equalsIgnoreCase ("sprite")) getHashMapSprite ().put (key[0], spriteSheetLoader (file.getPath (), 128, 150));
			}
			else {
				if (!file.isHidden ()) {
					imagesLoader (file.getPath ());
				}
			}
		}
	}
	
	private static SpriteSheet spriteSheetLoader (String spriteSheetPath, int tileWidth, int tileHeight) {
		SpriteSheet spriteSheet = null;
		try {
			spriteSheet = new SpriteSheet (spriteSheetPath, tileWidth, tileHeight);
			System.out.println ("dibe");
		}
		catch (SlickException e) {
			System.out.println ("No spritesheet found !");
			e.printStackTrace ();
		}
		return spriteSheet;
	}

	/**
	 * @param hashMapSprite the hashMapSprite to set
	 */
	public static void setHashMapSprite (HashMap<String, SpriteSheet> hashMapSprite) {
		MonsterLoader.hashMapSprite = hashMapSprite;
	}

	/**
	 * @return the hashMapSprite
	 */
	public static HashMap<String, SpriteSheet> getHashMapSprite () {
		return hashMapSprite;
	}
}
