// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * 
 */
package loader.events;

import java.io.File;
import java.util.HashMap;

import model.events.Event;
import model.events.NpcEvent;
import model.events.PortalEvent;
import modules.EventReader;
import modules.NpcReader;
import modules.PortalEventReader;

/**
 * @author Jérémy Chevrier
 * 
 */
public class EventXMLLoader {
	///////////////
	// Attributs //
	///////////////
	/**
	 * Liste de toutes les images d'une catégorie
	 */
	private static HashMap<String, Event> hashMapEvent = new HashMap<String, Event> ();
	/**
	 * La liste des chemins de toutes les fichiers à charger
	 */
	private static File[] list;

	/**
	 * Charge touts les events du jeu
	 */
	public static void eventsLoader (String folderPath) {
		File directory = new File (folderPath); // On fixe le dossier contenant les maps
		if (directory.isDirectory ()) setList (directory.listFiles ()); // On charge la liste des fichiers contenu dans ce dossier
		for (File file : getList ()) {
			if (!file.isDirectory ()) { // Si c'est un fichier, alors on le charge
				String[] key = file.getPath ().split ("_")[1].split (".xdat"); // On récupère la clé
				getHashMapEvent ().put (key[0], eventXMLLoader (file.getPath (), file.getParentFile ().getName ())); // On charge la map et on la met dans une hashTable
			}
			else {
				if (!file.isHidden ()) {
					eventsLoader (file.getPath ());
				}
			}
		}
	}

	/**
	 * Charge une image à partir d'un fichier
	 * 
	 * @param filePath
	 *            Le chemin du fichier
	 * @return une image
	 */
	public static Event eventXMLLoader (String filePath, String folder) {
		Event event = null;
		if (folder.equalsIgnoreCase ("basicEvents")) event = (Event) new EventReader ().getGameObject (filePath);
		else if (folder.equalsIgnoreCase ("portalEvents")) event = (PortalEvent) new PortalEventReader ().getGameObject (filePath);
		else if (folder.equalsIgnoreCase ("staticNPCEvents") || folder.equalsIgnoreCase ("dynamicNPCEvents")) event = (NpcEvent) new NpcReader ().getGameObject (filePath);
		return event;
	}

	/**
	 * @param hashMapEvent
	 *            the hashMapEvent to set
	 */
	public static void setHashMapEvent (HashMap<String, Event> hashMapEvent) {
		EventXMLLoader.hashMapEvent = hashMapEvent;
	}

	/**
	 * @return the hashMapEvent
	 */
	public static HashMap<String, Event> getHashMapEvent () {
		return hashMapEvent;
	}

	/**
	 * @param list
	 *            the list to set
	 */
	public static void setList (File[] list) {
		EventXMLLoader.list = list;
	}

	/**
	 * @return the list
	 */
	public static File[] getList () {
		return list;
	}
}
