// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * Le modèle du jeu
 */
package model;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;

import loader.MapLoader;
import loader.events.EventXMLLoader;
import model.characters.Hero;
import model.events.PortalEvent;
import model.maps.Maps;
import model.menu.Menu;
import enumeration.Moving;

/**
 * Modèle du jeu, Comporte les données
 * 
 * @author Jérémy Chevrier
 * @version 0.1.1 : 2011-01-17, 8:31pm<br>
 *          0.1.2 : 2011-01-19, 8:50pm<br>
 *          0.1.3 : 2011-01-27, 4:53pm
 */
public class Model {

	///////////////
	// Attributs //
	///////////////
	/**
	 * Objet de type <code>Hero</code>
	 */
	private Hero hero;
	/**
	 * Objet de type <ocde>Maps</code>
	 */
	private Maps map;
	/**
	 * Liste de toutes les maps
	 */
	private HashMap<String, Maps> listMaps = new HashMap<String, Maps> ();
	/**
	 * Savoir si on est sur un écrant de map ou non
	 */
	private boolean outMap;
	/**
	 * Un menu
	 */
	private Menu menu;
	/**
	 * Le tableau contenant les variables du jeu
	 */
	private int[] gameVar;
	/**
	 * La longueur du tableau des variables de jeu
	 */
	private final int LENGTHGAMEVAR = 10000;
	
	/**
	 * 
	 */
	private final int TIMEMONSTER = 600;

	//////////////////
	// Constructeur //
	//////////////////
	/**
	 * Construit un objet de type <ode>Model</code>
	 */
	public Model () {}

	//////////////
	// Méthodes //
	//////////////
	public void loadModel (boolean needingLoading, String name) {
		gameVar = new int[LENGTHGAMEVAR];
		for (int i = 0; i < gameVar.length; i++)
			gameVar[i] = 0;
		gameVar[2] = 428;
		gameVar[3] = 490;
		gameVar[6] = 140;
		gameVar[7] = 140;
		String keyMap = "0-0";
		if (needingLoading) {
			ArrayList<String[]> listVariables = SaveLoad.loadGame (name);
			for (String[] strArray : listVariables) {
				if (!strArray[0].equals ("4")) gameVar[Integer.parseInt (strArray[0])] = Integer.parseInt (strArray[1]);
				else keyMap = strArray[1];
			}
		}
		if (name.split (".").length > 1) setHero (new Hero (gameVar[2], gameVar[3], gameVar[10], gameVar[1], gameVar[6], gameVar[7], gameVar[8], gameVar[9], gameVar[15], gameVar[16], name.split ("_")[0]));
		else setHero (new Hero (gameVar[2], gameVar[3], gameVar[10], gameVar[1], gameVar[6], gameVar[7], gameVar[8], gameVar[9], gameVar[15], gameVar[16], name));
		this.loadMaps (); // On charge toutes les maps
		this.setMap (this.getListMaps ().get (keyMap)); // On place la carte de départ qu'il faut
		this.setOutMap (false);
	}

	/**
	 * Clone du Héro
	 */
	public Hero getTmpHero () {
		return this.getHero ();
	}

	/**
	 * Charge toutes les maps du jeu
	 */
	private void loadMaps () {
		for (Iterator<String> i = MapLoader.getHashTableMap ().keySet ().iterator (); i.hasNext ();) {
			String key = i.next ();
			listMaps.put (key, new Maps (key, MapLoader.getHashTableMap ().get (key), gameVar));
		}
	}

	/**
	 * Change de carte
	 * 
	 * @param key
	 *            La clé de la map suivante
	 * @param moving
	 *            Direction du joueur
	 */
	public void changeMap (String key, Moving moving) {
		this.setMap (this.getListMaps ().get (key));
		switch (moving) {
			case UP:
				this.getHero ().changeCoor (0, 576);
				if (this.getHero ().getNbStep () > 600) this.getHero ().setNbStep (0); 
				break;
			case DOWN:
				this.getHero ().changeCoor (0, -576);
				if (this.getHero ().getNbStep () > 600) this.getHero ().setNbStep (0);
				break;
			case LEFT:
				this.getHero ().changeCoor (800, 0);
				if (this.getHero ().getNbStep () > 600) this.getHero ().setNbStep (0);
				break;
			case RIGHT:
				this.getHero ().changeCoor (-800, 0);
				if (this.getHero ().getNbStep () > 600) this.getHero ().setNbStep (0);
				break;
		}
	}

	/**
	 * @param key
	 *            la clé de l'event
	 */
	public void comeInHouse (String key) {
		this.setMap (this.getListMaps ().get (((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getKeyMap ()));
		this.getHero ().changeCoorAbs (((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getHeroXLocation (), ((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getHeroYLocation ());
		this.getHero ().setOnMap (false);
	}

	public void comeOutHouse (String key) {
		this.setMap (this.getListMaps ().get (((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getKeyMap ()));
		this.getHero ().changeCoorAbs (((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getHeroXLocation (), ((PortalEvent) EventXMLLoader.getHashMapEvent ().get (key)).getHeroYLocation ());
		this.getHero ().setOnMap (true);
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param hero
	 *            the hero to set
	 */
	public void setHero (Hero hero) {
		this.hero = hero;
	}

	/**
	 * @return the hero
	 */
	public Hero getHero () {
		return hero;
	}

	/**
	 * @param map
	 *            the map to set
	 */
	public void setMap (Maps map) {
		this.map = map;
	}

	/**
	 * @return the map
	 */
	public Maps getMap () {
		return map;
	}

	/**
	 * @param listMaps
	 *            the listMaps to set
	 */
	public void setListMaps (HashMap<String, Maps> listMaps) {
		this.listMaps = listMaps;
	}

	/**
	 * @return the listMaps
	 */
	public HashMap<String, Maps> getListMaps () {
		return listMaps;
	}

	/**
	 * @param outMap
	 *            the outMap to set
	 */
	public void setOutMap (boolean outMap) {
		this.outMap = outMap;
	}

	/**
	 * @return the outMap
	 */
	public boolean isOutMap () {
		return outMap;
	}

	/**
	 * @param menu
	 *            the menu to set
	 */
	public void setMenu (Menu menu) {
		this.menu = menu;
	}

	/**
	 * @return the menu
	 */
	public Menu getMenu () {
		return menu;
	}

	/**
	 * @return the gameVar
	 */
	public int[] getGameVar () {
		return gameVar;
	}

	/**
	 * @param gameVar
	 *            the gameVar to set
	 */
	public void setGameVar (int[] gameVar) {
		this.gameVar = gameVar;
	}

	/**
	 * @return the tIMEMONSTER
	 */
	public int getTIMEMONSTER () {
		return TIMEMONSTER;
	}
}
