// ////////////////////////////////////////////////////////////////////////
// Copyright � 2011 J�r�my Chevrier, Hugo Buffet, J�r�my Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details. //
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

package model.battle;

import model.Skill;
import model.characters.Character;
import model.characters.Hero;
import model.characters.Spirit;

/**
 * @author Hugo Buffet
 * @version 0.1.0 : 2011-02-17, 11:48pm</br>
 * @version 0.1.1 : 2011-03-15, 6:01pm</br>
 * @version 0.1.2 : 2011-03-15, 8:04pm</br>
 * 
 */
public class BattleEngine {

	private static Hero hero;
	private static Spirit spirit1;
	private static Spirit spirit2;
	private static Spirit spirit3;

	/**
	 * Cette méthode décrit un round de combat, c'est à dire lorsque l'un des
	 * personnages utilise une compétence sur une cible.
	 * 
	 * @param skill
	 *            Compétence utilisée dans ce round.
	 * @param attacking
	 *            Utilisateur du sort.
	 * @param target
	 *            Cible de la compétence.
	 * @return Le succès de la compétence.
	 */
	public static String battleRound (Skill skill, Character attacking, Character target) {
		
		if (testSuccess (skill)) {
			
			String sentence = null;
			
			if (skill.getAttBonus () != 0) {
				attBonus (skill, attacking, target);
			}
			if (skill.getDefBonus () != 0) {
				defBonus (skill, attacking, target);
			}
			if (skill.getHpBonus () != 0) {
				hpBonus (skill, attacking, target);
			}
			if (skill.getAuraBonus () != 0 && isHero (target)) {
				auraBonus (skill, (Hero) target);
			}
			if (skill.getPowerBonus () != 0 && isSpirit (target)) {
				powerBonus (skill, attacking, (Spirit) target);
			}
			if (skill.getAttMalus () != 0) {
				attMalus (skill, attacking, target);
			}
			if (skill.getDefMalus () != 0) {
				defMalus (skill, attacking, target);
			}
			if (skill.getDamages () != 0) {
				int damages = attackValue (skill, attacking) - defenseValue (target);
				if(damages<0)damages = 0;
				
				target.setHp (target.getHp () - damages);
				
				sentence = attacking.getName ().split ("_")[0]+" touche "+ target.getName ().split ("_")[0] +" avec \n"+skill.getName ()+" et lui inflige "+damages+" \nde degats !";
			}

			return sentence;
		}
		
		return attacking.getName ().split ("_")[0]+" a rate son coup !";

	}

	/**
	 * Cette méthode teste la réussite de la compétence en fonction de son
	 * attribut Hitsuccess.
	 * 
	 * @param skill
	 *            Compétence à tester.
	 * @return le résultat du succès de la compétence.
	 */
	public static boolean testSuccess (Skill skill) {
		int success = (int) (Math.random () * 100);

		if (skill.getHitSuccess () >= success) { return true; }

		return false;
	}

	// Méthodes de test de classe
	/**
	 * Cette méthode teste si un character est le Héro ou non.
	 * 
	 * @param charact
	 *            Le character à analyser.
	 * @return si le character est de type Hero.
	 */
	public static boolean isHero (Character charact) {
		if (charact.getClass ().equals (Hero.class)) return true;
		return false;
	}

	/**
	 * Cette méthode teste si un character est un esprit ou non.
	 * 
	 * @param charact
	 *            Le character à analyser.
	 * @return si le character est de type Spirit.
	 */
	public static boolean isSpirit (Character charact) {
		if (charact.getClass ().equals (Spirit.class)) return true;
		return false;
	}

	// M�thodes sur les statistiques du h�ro et des esprits en d�but et
	// fin de combat

	/**
	 * Cette m�thode permet de sauvegarder les statistiques du hero dans un hero
	 * temporaire
	 * 
	 * @param hero
	 *            Le hero
	 */
	public static void saveStatsHero (Hero hero) {

	}

	/**
	 * Cette methode permet de restaurer les statistiques du hero a partir de la
	 * sauvegarde
	 * 
	 * @param hero
	 *            Le hero
	 */
	public static void restoreStatsHero (Hero hero) {
		hero.setAtt (getHero ().getAtt ());
		hero.setDef (getHero ().getDef ());
		hero.setAura (getHero ().getAura ());
	}

	/**
	 * Cette m�thode permet de sauvegarder les statistiques du premier esprit
	 * dans un esprit temporaire
	 * 
	 * @param spirit
	 *            Le premier esprit
	 */
	public static void saveStatsSpirit1 (Spirit spirit) {

	}

	/**
	 * Cette methode permet de restaurer les statistiques du second esprit a
	 * partir de la sauvegarde
	 * 
	 * @param spirit
	 *            Le second esprit
	 */
	public static void restoreStatsSpirit1 (Spirit spirit) {
		spirit.setAtt (getSpirit1 ().getAtt ());
		spirit.setDef (getSpirit1 ().getDef ());
		spirit.setPower (getSpirit1 ().getPower ());
	}

	/**
	 * Cette m�thode permet de sauvegarder les statistiques du second esprit
	 * dans un esprit temporaire
	 * 
	 * @param spirit
	 *            Le second esprit
	 */
	public static void saveStatsSpirit2 (Spirit spirit) {

	}

	/**
	 * Cette methode permet de restaurer les statistiques du troisieme esprit a
	 * partir de la sauvegarde
	 * 
	 * @param spirit
	 *            Le troisieme esprit
	 */
	public static void restoreStatsSpirit2 (Spirit spirit) {
		spirit.setAtt (getSpirit2 ().getAtt ());
		spirit.setDef (getSpirit2 ().getDef ());
		spirit.setPower (getSpirit2 ().getPower ());
	}

	/**
	 * Cette m�thode permet de sauvegarder les statistiques du troisieme esprit
	 * dans un esprit temporaire
	 * 
	 * @param spirit
	 *            Le troisieme esprit
	 */
	public static void saveStatsSpirit3 (Spirit spirit) {

	}

	/**
	 * Cette methode permet de restaurer les statistiques du premier esprit a
	 * partir de la sauvegarde
	 * 
	 * @param spirit
	 *            Le premier esprit
	 */
	public static void restoreStatsSpirit3 (Spirit spirit) {
		spirit.setAtt (getSpirit3 ().getAtt ());
		spirit.setDef (getSpirit3 ().getDef ());
		spirit.setPower (getSpirit3 ().getPower ());
	}

	// Méthode d'amplification pour les esprits
	/**
	 * Permet de récupérer la valeur d'amplification de l'utilisation du sort
	 * par un esprit.
	 * 
	 * @param spirit
	 *            Esprit qui utilise la compétente.
	 * @param value
	 *            Valeur initiale de la compétence lancée.
	 * @return l'ajout résultant de l'amplification.
	 */
	public static int valueAmpli (Spirit spirit, int value) {
		// constantes d'équilibrage
		double constPower = 0.25;
		double constAura = 0.05;

		return (int) (constPower * spirit.getPower () * value);
		//return (int) (constPower * spirit.getPower () * value + constAura * hero.getAura () * value);
	}

	/**
	 * Permet de de faire osciller la valeur final d'une action à +- un certain
	 * pourcentage aléatoire de sa valeur initiale.
	 * 
	 * @param value
	 *            Valeur initiale.
	 * @return valeur modifiée.
	 */
	public static int randomVal (int value) {
		// constante d'équilibrage
		double constValue = 0.20;

		double test = Math.random () * 100;

		if (test < 50) {
			value += (int) (Math.random () * constValue);
		}
		else {
			value -= (int) (Math.random () * constValue);
		}

		return value;
	}

	// M�thodes sur les valeurs de jets d'attaque et de d�fense

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "domages"
	 * 
	 * @param skill
	 *            Competence utiliee
	 * @param attacking
	 *            Utilisateur de la competence
	 * @return la valeur calculee
	 */
	public static int attackValue (Skill skill, Character attacking) {
		int attackValue = skill.getDamages () + attacking.getAtt ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (attacking)) attackValue += valueAmpli ((Spirit) attacking, attackValue);

		return randomVal (attackValue);
	}

	public static int defenseValue (Character defender) {
		return defender.getDef ();
	}

	// M�thodes sur les bonus

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "bonus d'attaque"
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void attBonus (Skill skill, Character caster, Character target) {
		int initAttValue = target.getAtt ();
		int valBonus = skill.getAttBonus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valBonus += valueAmpli ((Spirit) caster, valBonus);

		target.setAtt (randomVal (initAttValue + valBonus));
	}

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "bonus de defense"
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void defBonus (Skill skill, Character caster, Character target) {
		int initDefValue = target.getDef ();
		int valBonus = skill.getDefBonus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valBonus += valueAmpli ((Spirit) caster, valBonus);

		target.setDef (randomVal (initDefValue + valBonus));
	}

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "soin"
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void hpBonus (Skill skill, Character caster, Character target) {
		int initHpValue = target.getHp ();
		int valBonus = skill.getHpBonus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valBonus += valueAmpli ((Spirit) caster, valBonus);

		int finalValue = randomVal (initHpValue + valBonus);

		if (finalValue > target.getMaxHP ()) finalValue = target.getMaxHP ();

		target.setHp (finalValue);
	}

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "bonus d'aura". Note : utilisable seulement sur un
	 * Charactere de type Hero.
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void auraBonus (Skill skill, Hero target) {
		int initAuraValue = target.getAura ();

		target.setAtt (randomVal (initAuraValue + skill.getHpBonus ()));
	}

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "bonus de puissance". Note : utilisable seulement sur
	 * un Charactere de type Hero.
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void powerBonus (Skill skill, Character caster, Spirit target) {
		int initPowerValue = target.getPower ();
		int valBonus = skill.getPowerBonus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valBonus += valueAmpli ((Spirit) caster, valBonus);

		target.setPower (randomVal (initPowerValue + valBonus));
	}

	// M�thodes sur les malus

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "malus d'attaque"
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void attMalus (Skill skill, Character caster, Character target) {
		int initAttValue = target.getAtt ();
		int valBonus = skill.getAttMalus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valBonus += valueAmpli ((Spirit) caster, valBonus);

		target.setAtt (randomVal (initAttValue - valBonus));
	}

	/**
	 * M�thode calculant la valeur d'attaque lors de l'utilisation d'une
	 * comp�tence de type "bonus de d�fense"
	 * 
	 * @param skill
	 *            Competence utilisee
	 * @param caster
	 *            Utilisateur de la competence
	 * @param target
	 *            Cible de la competence
	 */
	public static void defMalus (Skill skill, Character caster, Character target) {
		int initDefValue = target.getDef ();
		int valMalus = skill.getDefMalus ();

		// Si le lanceur est un esprit, alors on amplifie la valeur
		if (isSpirit (caster)) valMalus += valueAmpli ((Spirit) caster, valMalus);

		target.setDef (randomVal (initDefValue - valMalus));
	}

	// Accessors
	/**
	 * @param hero
	 */
	public static void setHero (Hero hero) {
		BattleEngine.hero = hero;
	}

	/**
	 * @return the hero
	 */
	public static Hero getHero () {
		return hero;
	}

	/**
	 * @return the spirit1
	 */
	public static Spirit getSpirit1 () {
		return spirit1;
	}

	/**
	 * @param spirit1
	 *            the spirit1 to set
	 */
	public static void setSpirit1 (Spirit spirit1) {
		BattleEngine.spirit1 = spirit1;
	}

	/**
	 * @return the spirit2
	 */
	public static Spirit getSpirit2 () {
		return spirit2;
	}

	/**
	 * @param spirit2
	 *            the spirit2 to set
	 */
	public static void setSpirit2 (Spirit spirit2) {
		BattleEngine.spirit2 = spirit2;
	}

	/**
	 * @return the spirit3
	 */
	public static Spirit getSpirit3 () {
		return spirit3;
	}

	/**
	 * @param spirit3
	 *            the spirit3 to set
	 */
	public static void setSpirit3 (Spirit spirit3) {
		BattleEngine.spirit3 = spirit3;
	}

}
