// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * Le modèle du jeu
 */
package model.characters;

import java.util.ArrayList;

import loader.characters.HeroLoader;
import model.Skill;

import org.newdawn.slick.Animation;
import org.newdawn.slick.SpriteSheet;

import enumeration.Moving;

/**
 * <code>Hero</code> est la classe repr&eacute; le h&eacute;ro.
 * 
 * @author J&eacute;r&eacute;my Chevrier
 * @version 0.1.1 : 2011-01-16, 4:14pm<br>
 *          0.1.2 : 2011-01-19, 8:41pm<br>
 */
public class Hero extends Character {

	//////////////////////
	// Attributs du héro//
	//////////////////////
	/**
	 * Abscisse du joueur
	 */
	private int x;
	/**
	 * Ordonn&eacute;e du joueur
	 */
	private int y;
	/**
	 * Largeur du joueur
	 */
	private int width;
	/**
	 * Hauteur des pieds du joueur
	 */
	private int height;
	/**
	 * La feuille de sprites du perso
	 */
	private SpriteSheet spriteSheet;
	/**
	 * liste des animations du personnage
	 */
	private ArrayList<Animation> animationMove;
	/**
	 * AnimationMap courante
	 */
	private Animation currentAnimation;
	/**
	 * Direction courante du joueur
	 */
	private int currentDirection;
	/**
	 * Le joueur est-il en train de marcher
	 */
	private boolean walking;
	/**
	 * le joueur est sur une map ou un intérieur
	 */
	private boolean onMap;
	/**
	 * l'aura du joueur
	 */
	private int aura;
	/**
	 * Argent du joueur
	 */
	private int money;
	private ArrayList<Spirit> listSpirit;
	private int nbStep;

	//////////////////
	// Constructeur //
	//////////////////
	/**
	 * Cr&eacute; une instance du h&eacute;ro
	 * 
	 * @param x
	 *            Abscisse du joueur
	 * @param y
	 *            Ordonn&eacute;e du joueur
	 */
	public Hero (int x, int y, int aura, int money, int hp, int maxHP, int att, int def, int lvl, int exp, String name) {
		super (null, null, "Le héro", hp, maxHP, att, def, lvl, exp, name, null);
		// On fixe les coordonnées du joueur
		this.setX (x);
		this.setY (y);
		this.setAura (aura);
		this.setMoney (money);
		Skill skill1 = new Skill ("skill4", null, "Coup d'epee", 1, 0, 0, 0, 0, 0, 0, 0, 10, "Attack", 90, "Coup d'epee");
		Skill skill2 = new Skill ("skill5", null, "Coup eclair", 2, 0, 0, 0, 0, 0, 0, 0, 15, "Attack", 80, "Coup eclair");
		Skill skill3 = new Skill ("skill6", null, "boule de feu", 1, 0, 0, 0, 0, 0, 0, 0, 18, "Attack", 90, "Boule de feu");
		Skill skill4 = new Skill ("skill7", null, "Eclair", 2, 0, 0, 0, 0, 0, 0, 0, 25, "Attack", 80, "Eclair de feu ");
		//Skill skill5 = new Skill ("skill8", null, "Meteor de feu", 4, 0, 0, 0, 0, 0, 0, 0, 60, "Attack", 80, "Meteor de feu ");
		ArrayList<Skill> listSkills = new ArrayList<Skill> ();
		listSkills.add (skill1);
		listSkills.add (skill2);
		ArrayList<Skill> listSkills2 = new ArrayList<Skill> ();
		listSkills2.add (skill3);
		listSkills2.add (skill4);
		//listSkills2.add (skill5);
		this.setSkills (listSkills);
		this.setCurrentDirection (Moving.DOWN); // On fixe la direction du joueur (par défaut, vers le bas)
		this.setWalking (false);
		// On charge les images du joueur
		HeroLoader.imagesLoader ();
		// On charge la feuille de sprites
		this.setSpriteSheet (HeroLoader.getSpriteSheetBase ());
		this.loadAnimations ();
		this.setCurrentAnimation (this.getAnimationMove ().get (Moving.DOWN.getDirection ()));
		this.startAnimation ();
		this.setOnMap (true);
		listSpirit = new ArrayList<Spirit> ();
		listSpirit.add (new Spirit ("sprit", null, "Esprit", 80, 80, 15, 10, 1, 1, "Esprit", listSkills2, 4, "gentil"));
		this.setNbStep (0);
	}

	//////////////
	// Méthodes //
	//////////////
	/**
	 * Démarre l'animation du perso (à chaque changement de direction)
	 */
	public void startAnimation () {
		currentAnimation.start ();
	}

	/**
	 * Stop l'animation du personnage
	 */
	public void stopAnimation () {
		currentAnimation.stop ();
	}

	/**
	 * Met en pause l'animation du personnage
	 */
	public void pauseAnimation () {
		this.setWalking (false);
	}

	/**
	 * Dessine l'animation du personnage
	 */
	public void drawAnimation () {
		if (this.isWalking ()) currentAnimation.draw (x, y - 28);
		else currentAnimation.getImage (1).draw (x, y - 28);
	}

	/**
	 * Relance l'animation
	 */
	public void restartAnimation (Moving moving) {
		this.stopAnimation ();
		this.setCurrentDirection (moving);
		this.setCurrentAnimation (this.getAnimationMove ().get (moving.getDirection ()));
		this.startAnimation ();
	}

	/**
	 * Charge toutes les animations du personnage
	 */
	private void loadAnimations () {
		// On charge la liste des animations
		animationMove = new ArrayList<Animation> ();
		for (int i = 0; i < 4; i++)
			animationMove.add (i, new Animation (spriteSheet, 0, i, 2, i, true, 200, true));
	}

	// Fin partie animation

	/**
	 * Change les coordonnées du joueur par rapport à une fin de map
	 * 
	 * @param x
	 *            A ajouter à l'abscisse actuelle du héro
	 * @param y
	 *            A ajouter à l'ordonnée actuelle du héro
	 */
	public void changeCoor (int x, int y) {
		this.setX (this.getX () + x);
		this.setY (this.getY () + y);
	}
	
	public void changeCoorAbs (int x, int y) {
		this.setX (x);
		this.setY (y);
	}
	
	/**
	 * Récupère la direction
	 * 
	 * @param direction
	 *            la direction en int
	 * @return une partie de l'énumération
	 */
	public Moving getMoving () {
		if (currentDirection == 0) return Moving.DOWN;
		else if (currentDirection == 1) return Moving.LEFT;
		else if (currentDirection == 2) return Moving.RIGHT;
		else if (currentDirection == 3) return Moving.UP;
		else return Moving.UP;
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param x
	 *            the x to set
	 */
	public void setX (int x) {
		this.setNbStep (this.getNbStep () + Math.abs (x - this.x));
		this.x = x;
	}

	/**
	 * @return the x
	 */
	public int getX () {
		return x;
	}

	/**
	 * @param y
	 *            the y to set
	 */
	public void setY (int y) {
		this.setNbStep (this.getNbStep () + Math.abs (y - this.y));
		this.y = y;
	}

	/**
	 * @return the y
	 */
	public int getY () {
		return y;
	}

	/**
	 * @param width
	 *            the width to set
	 */
	public void setWidth (int width) {
		this.width = width;
	}

	/**
	 * @return the width
	 */
	public int getWidth () {
		return width;
	}

	/**
	 * @param height
	 *            the height to set
	 */
	public void setHeight (int height) {
		this.height = height;
	}

	/**
	 * @return the height
	 */
	public int getHeight () {
		return height;
	}

	/**
	 * @param spriteSheet
	 *            the spriteSheet to set
	 */
	public void setSpriteSheet (SpriteSheet spriteSheet) {
		this.spriteSheet = spriteSheet;
	}

	/**
	 * @return the spriteSheet
	 */
	public SpriteSheet getSpriteSheet () {
		return spriteSheet;
	}

	/**
	 * @param animationMove
	 *            the animationMove to set
	 */
	public void setAnimationMove (ArrayList<Animation> animationMove) {
		this.animationMove = animationMove;
	}

	/**
	 * @return the animationMove
	 */
	public ArrayList<Animation> getAnimationMove () {
		return animationMove;
	}

	/**
	 * @param moving
	 *            the curentDirection to set
	 */
	public void setCurrentDirection (Moving moving) {
		this.currentDirection = moving.getDirection ();
	}

	/**
	 * @return the curentDirection
	 */
	public int getCurrentDirection () {
		return currentDirection;
	}

	/**
	 * @param currentAnimation
	 *            the currentAnimation to set
	 */
	public void setCurrentAnimation (Animation currentAnimation) {
		this.currentAnimation = currentAnimation;
	}

	/**
	 * @return the currentAnimation
	 */
	public Animation getCurrentAnimation () {
		return currentAnimation;
	}

	/**
	 * @param walking
	 *            the walking to set
	 */
	public void setWalking (boolean walking) {
		this.walking = walking;
	}

	/**
	 * @return the walking
	 */
	public boolean isWalking () {
		return walking;
	}

	/**
	 * @param onMap the onMap to set
	 */
	public void setOnMap (boolean onMap) {
		this.onMap = onMap;
	}

	/**
	 * @return the onMap
	 */
	public boolean isOnMap () {
		return onMap;
	}

	/**
	 * @param aura the aura to set
	 */
	public void setAura (int aura) {
		this.aura = aura;
	}

	/**
	 * @return the aura
	 */
	public int getAura () {
		return aura;
	}

	/**
	 * @param money the money to set
	 */
	public void setMoney (int money) {
		this.money = money;
	}

	/**
	 * @return the money
	 */
	public int getMoney () {
		return money;
	}

	/**
	 * @return the listSpirit
	 */
	public ArrayList<Spirit> getListSpirit () {
		return listSpirit;
	}

	/**
	 * @param listSpirit the listSpirit to set
	 */
	public void setListSpirit (ArrayList<Spirit> listSpirit) {
		this.listSpirit = listSpirit;
	}

	/**
	 * @param nbStep the nbStep to set
	 */
	public void setNbStep (int nbStep) {
		this.nbStep = nbStep;
	}

	/**
	 * @return the nbStep
	 */
	public int getNbStep () {
		return nbStep;
	}
}
