// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * Le modèle du jeu
 */
package model.maps;

import java.awt.Point;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.Map;

import loader.characters.NpcsStaticLoader;
import loader.events.EventXMLLoader;
import model.GameObject;
import model.events.Event;
import model.events.NpcEvent;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Image;
import org.newdawn.slick.tiled.TiledMap;

/**
 * <code>Maps</code> repr&eacute;sente une carte du jeu
 * 
 * @author Jérémy Chevrier
 * @version 0.1.1 : 2011-01-17, 3:49pm<br>
 *          0.1.2 : 2011-01-19, 8:50pm<br>
 *          0.1.3 : 2011-01-19, 9:44pm<br>
 *          0.1.4 : 2011-01-
 */
public class Maps extends GameObject {

	///////////////
	// Attributs //
	///////////////
	/**
	 * La liste des éléments bloquant de la map
	 */
	private ArrayList<Point> blocked = new ArrayList<Point> ();
	/**
	 * La liste des éléments ralentissant de la map
	 */
	private ArrayList<Point> slowlier = new ArrayList<Point> ();
	/**
	 * La liste de toutes les coordonnées des évènements
	 */
	private HashMap<String, Point> eventPoint = new HashMap<String, Point> ();
	/**
	 * La map version xml chargée
	 */
	private TiledMap map;
	/**
	 * La couche de land
	 */
	private int land;
	/**
	 * La couche intermédiaire premier niveau
	 */
	private int medium1;
	/**
	 * La couche intermédiaire second niveau
	 */
	private int medium2;
	/**
	 * La couche du dessus, premier niveau
	 */
	private int top1;
	/**
	 * La couche du dessus, second niveau
	 */
	private int top2;
	/**
	 * liste des animations du personnage
	 */
	private ArrayList<Animation> animationMove;
	/**
	 * On dessine l'animation ou pas
	 */
	private boolean drawingAnimation = true;
	/**
	 * La liste des animations
	 */
	private ArrayList<AnimationMap> listAnimation = new ArrayList<AnimationMap> ();
	/**
	 * La liste des animations des NPCS
	 */
	private ArrayList<AnimationNPCMap> listNPCMaps = new ArrayList<AnimationNPCMap> ();
	/**
	 * Le tableau des variables
	 */
	private int[] gameVar;
	
	/**
	 * 
	 */
	private String fightZone;

	//////////////////
	// Constructeur //
	//////////////////
	/**
	 * Construit un objet de type Maps
	 * 
	 * @param key
	 *            L'identifiant de la carte
	 * @param map
	 *            La map version xml chargée
	 */
	public Maps (String key, TiledMap map, int[] gameVar) {
		super (key, null, "A piece of map", null);
		this.map = map;
		this.getCollisionMap (); // On charge les block blocant
		this.loadLayers (); // On charge les différentes couches
		this.setGameVar (gameVar);
	}

	/**
	 * Dessine les animations
	 */
	public void drawAnimation () {
		for (AnimationMap array : this.getListAnimation ()) {
			array.drawAnimation ();
		}
	}

	public void decAnimation (int delta) {
		for (AnimationMap array : this.getListAnimation ()) {
			array.setDeltaInc (array.getDeltaInc () - delta);
		}
	}

	/**
	 * Charge toutes les couches de la map
	 */
	public void loadLayers () {
		land = map.getLayerIndex ("land");
		medium1 = map.getLayerIndex ("medium1");
		medium2 = map.getLayerIndex ("medium2");
		top1 = map.getLayerIndex ("top1");
		top2 = map.getLayerIndex ("top2");
	}

	/**
	 * Dessine à l'écran la partie basse (en couche) de la carte. Ce qui est
	 * dessiné sous le héro
	 */
	public void drawLowLayerMap () {
		map.render (0, 0, land);
		map.render (0, 0, medium1);
		map.render (0, 0, medium2);
	}

	/**
	 * Dessine à l'écran la partie haute (en couche) de la carte. Ce qui est
	 * dessiné sur le héro
	 */
	public void drawHighLayerMap () {
		map.render (0, 0, top1);
		map.render (0, 0, top2);
		//				map.render (0, 0, map.getLayerIndex ("block")); // Dessine les objets représentant les cases bloquantes
	}

	/**
	 * Dessine le bas du PNJ sur la map
	 */
	public void drawNpc () {
		for (Map.Entry<String, Point> e : this.getEventPoint ().entrySet ()) {
			Event event = EventXMLLoader.getHashMapEvent ().get (e.getKey ());
			if (event.getEventType ().equalsIgnoreCase ("npcS")) {
				if (event instanceof NpcEvent) {
					int valueGameVar = ((NpcEvent) event).getValueArrayIndex ();
					if (valueGameVar > 9000) {
						if (this.getGameVar ()[valueGameVar] == 0) {
							NpcsStaticLoader.getHashMapImage ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent ());
						}
						else if (this.getGameVar ()[valueGameVar] > 1) {}
					}
					else {
						NpcsStaticLoader.getHashMapImage ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent ());
					}
				}
				else {
					NpcsStaticLoader.getHashMapImage ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent ());
				}
			}
		}
		for (int i = 0; i < listNPCMaps.size (); i++)
			listNPCMaps.get (i).drawNPCAnimBottom ();
	}

	/**
	 * Dessine le haut du NPC sur la map
	 */
	public void drawNpcTop () {
		for (Map.Entry<String, Point> e : this.getEventPoint ().entrySet ()) {
			Event event = EventXMLLoader.getHashMapEvent ().get (e.getKey ());
			if (event.getEventType ().equalsIgnoreCase ("npcS")) {
				if (event instanceof NpcEvent) {
					int valueGameVar = ((NpcEvent) event).getValueArrayIndex ();
					if (valueGameVar > 9000) {
						if (this.getGameVar ()[valueGameVar] == 0) {
							if (NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()) != null) NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent () - NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).getHeight ());
						}
						else if (this.getGameVar ()[valueGameVar] > 1) {
							// Ici on affichera une image différente, pour l'instant non effectif
						}
					}
					else {
						if (NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()) != null) NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent () - NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).getHeight ());
					}
				}
				else {
					if (NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()) != null) NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).draw (event.getxEvent (), event.getyEvent () - NpcsStaticLoader.getHashMapImageTop ().get (e.getKey ()).getHeight ());
				}
			}
		}
		for (int i = 0; i < listNPCMaps.size (); i++)
			listNPCMaps.get (i).drawNPCAnimTop ();
	}

	/**
	 * Charge tout type de collision de la map
	 */
	public void getCollisionMap () {
		int block = map.getLayerIndex ("object");
		
		// On récupère si c'est une map à monstre
		fightZone = map.getMapProperty ("fightZone", "false");
		
		// build a collision map based on tile properties in the TileD map
		for (int x = 0; x < map.getWidth (); x++) {
			for (int y = 0; y < map.getHeight (); y++) {
				int tileID = map.getTileId (x, y, block);

				// Vrai Collision
				String value = map.getTileProperty (tileID, "blocked", "false");
				if ("true".equals (value)) blocked.add (new Point (x, y));

				// Ralentisseur
				String slowValue = map.getTileProperty (tileID, "speed", "false");
				if ("2".equals (slowValue)) slowlier.add (new Point (x, y));

				// évènements statiques
				String eventValue = map.getTileProperty (tileID, "type", "false");
				if (!"false".equals (eventValue)) {
					String globalKey = map.getTileProperty (tileID, "globalKey", "false");
					if (!"false".equals (globalKey)) {
						eventPoint.put (globalKey, new Point (x, y));
						EventXMLLoader.getHashMapEvent ().get (globalKey).setLocalkey (Integer.valueOf (map.getTileProperty (tileID, "localKey", "false")));
						if (eventValue.equalsIgnoreCase ("npcS")) {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (this.centerXElement (x, NpcsStaticLoader.getHashMapImage ().get (globalKey)));
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (this.centerYElement (y, NpcsStaticLoader.getHashMapImage ().get (globalKey)));
						}
						else if (eventValue.equalsIgnoreCase ("npcD")) {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (x * 32);
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (y * 32);
							listNPCMaps.add (new AnimationNPCMap (globalKey, x * 32, y * 32));
						}
						else {
							EventXMLLoader.getHashMapEvent ().get (globalKey).setxEvent (x);
							EventXMLLoader.getHashMapEvent ().get (globalKey).setyEvent (y);
						}
					}
				}
				
				// Les animations sur la map
				String animationValue = map.getTileProperty (tileID, "animation", "false");
				if (!"false".equals (animationValue)) {
					String mode = map.getTileProperty (tileID, "mode", "false");
					if (!"false".equals (mode)) listAnimation.add (new AnimationMap (animationValue, x * 32, y * 32, mode));
				}
			}
		}
	}

	/**
	 * Centre horizontalement un élément
	 * 
	 * @param x
	 *            La coordonnées en tiles
	 * @param img
	 *            L'image à centrer
	 * @return la coordonnée centrée
	 */
	private int centerXElement (int x, Image img) {
		return (x * 32) + ((32 - img.getWidth ()) / 2);
	}

	/**
	 * Centre verticalement un élément
	 * 
	 * @param y
	 *            La coordonnées en tiles
	 * @param img
	 *            L'image à centrer
	 * @return la coordonnées centrée
	 */
	private int centerYElement (int y, Image img) {
		return (y * 32) + ((32 - img.getHeight ()) / 2);
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @return the blocked
	 */
	public ArrayList<Point> getBlocked () {
		return blocked;
	}

	/**
	 * @param blocked
	 *            the blocked to set
	 */
	public void setBlocked (ArrayList<Point> blocked) {
		this.blocked = blocked;
	}

	/**
	 * @param slowlier
	 *            the slowlier to set
	 */
	public void setSlowlier (ArrayList<Point> slowlier) {
		this.slowlier = slowlier;
	}

	/**
	 * @return the slowlier
	 */
	public ArrayList<Point> getSlowlier () {
		return slowlier;
	}

	/**
	 * @param eventPoint
	 *            the eventPoint to set
	 */
	public void setEventPoint (HashMap<String, Point> eventPoint) {
		this.eventPoint = eventPoint;
	}

	/**
	 * @return the eventPoint
	 */
	public HashMap<String, Point> getEventPoint () {
		return eventPoint;
	}

	/**
	 * @param animationMove
	 *            the animationMove to set
	 */
	public void setAnimationMove (ArrayList<Animation> animationMove) {
		this.animationMove = animationMove;
	}

	/**
	 * @return the animationMove
	 */
	public ArrayList<Animation> getAnimationMove () {
		return animationMove;
	}

	/**
	 * @param drawingAnimation
	 *            the drawingAnimation to set
	 */
	public void setDrawingAnimation (boolean drawingAnimation) {
		this.drawingAnimation = drawingAnimation;
	}

	/**
	 * @return the drawingAnimation
	 */
	public boolean isDrawingAnimation () {
		return drawingAnimation;
	}

	/**
	 * @param listAnimation
	 *            the listAnimation to set
	 */
	public void setListAnimation (ArrayList<AnimationMap> listAnimation) {
		this.listAnimation = listAnimation;
	}

	/**
	 * @return the listAnimation
	 */
	public ArrayList<AnimationMap> getListAnimation () {
		return listAnimation;
	}

	/**
	 * @param listNPCMaps
	 *            the listNPCMaps to set
	 */
	public void setListNPCMaps (ArrayList<AnimationNPCMap> listNPCMaps) {
		this.listNPCMaps = listNPCMaps;
	}

	/**
	 * @return the listNPCMaps
	 */
	public ArrayList<AnimationNPCMap> getListNPCMaps () {
		return listNPCMaps;
	}

	/**
	 * @return the gameVar
	 */
	public int[] getGameVar () {
		return gameVar;
	}

	/**
	 * @param gameVar
	 *            the gameVar to set
	 */
	public void setGameVar (int[] gameVar) {
		this.gameVar = gameVar;
	}

	/**
	 * @param fightZone the fightZone to set
	 */
	public void setFightZone (String fightZone) {
		this.fightZone = fightZone;
	}

	/**
	 * @return the fightZone
	 */
	public String getFightZone () {
		return fightZone;
	}
}
