// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

/**
 * 
 */
package model.menu;

import java.util.ArrayList;
import java.util.Random;

import loader.FontLoader;
import loader.SoundsLoader;
import loader.characters.HeroLoader;
import loader.characters.MonsterLoader;
import loader.characters.MonstersPropLoader;
import loader.characters.SpiritLoader;
import loader.menus.DiversLoader;
import loader.menus.MenuLoader;
import model.Skill;
import model.battle.BattleEngine;
import model.characters.Character;
import model.characters.Monster;

import org.newdawn.slick.Animation;
import org.newdawn.slick.Color;
import org.newdawn.slick.state.StateBasedGame;
import org.newdawn.slick.state.transition.FadeInTransition;
import org.newdawn.slick.state.transition.FadeOutTransition;

import controller.Controller;

/**
 * @author jérémy Chevrier
 * 
 */
public class BattleMenu extends Menu {

	private ArrayList<ArrayList<Skill>> listArraySkills;
	private ArrayList<Skill> listNameSkillsHero;
	private int cursorChar = 0;
	private int cursorSkill = 0;
	private int nbSpirit = 0;
	private boolean charSelectionned = true;
	private Monster monster;
	private boolean userToPlay;
	private boolean endFight;
	private boolean endDisplayMonster;
	private boolean endDisplayHero;
	private boolean attack;
	private boolean attackM;
	private boolean affichMWin;
	private String messageFight;
	private Animation animationArrow, animationMonster, animationHero,
			animationSpirit, animationFight;
	private Character attacking;
	private StateBasedGame sbg;
	private int xFightAnim, yFightAnim;
	private String keyNumber;

	public BattleMenu (int[] gameVar, Controller controller, StateBasedGame sbg) {
		super (gameVar, controller);
		this.sbg = sbg;
		listArraySkills = new ArrayList<ArrayList<Skill>> ();
		listNameSkillsHero = new ArrayList<Skill> ();
		for (Skill skill : controller.getModel ().getHero ().getSkills ())
			listNameSkillsHero.add (skill);
		listArraySkills.add (listNameSkillsHero);
		this.setKeyNumber (randomKey ());
		monster = MonstersPropLoader.getHashMapMonsters ().get (this.getKeyNumber ());
		monster.getAtt ();
		affichMWin = false;
		endFight = false;
		attack = true;
		attackM = true;
		endDisplayMonster = false;
		endDisplayHero = false;
		userToPlay = true;
		this.loadAnimations ();
		nbSpirit = this.getController ().getModel ().getHero ().getListSpirit ().size ();
		attacking = this.getController ().getModel ().getHero ();
		SoundsLoader.getFight ().loop ();
	}

	private String randomKey () {
		Random r = new Random ();
		int keyNumber = r.nextInt (MonstersPropLoader.getHashMapMonsters ().size ()) + 1;
		if (keyNumber < 10) return "m00" + keyNumber;
		else if (keyNumber < 100) return "m0" + keyNumber;
		else if (keyNumber < 1000) return "m" + keyNumber;
		else return "m003";
	}

	private void loadAnimations () {
		// On charge la liste des animations
		animationArrow = new Animation (MenuLoader.getHashMapSprite ().get ("00"), 0, 0, 2, 0, true, 200, true);
		animationMonster = new Animation (MonsterLoader.getHashMapSprite ().get (this.getKeyNumber ()), 0, 0, 3, 0, true, 200, true);
		animationHero = new Animation (HeroLoader.getSpriteSheetHeroFight (), 0, 0, 1, 0, true, 200, true);
		animationSpirit = new Animation (SpiritLoader.getHashMapSprite ().get ("01"), 0, 0, 3, 0, true, 200, true);
		animationArrow.start ();
		animationMonster.start ();
		animationHero.start ();
		animationSpirit.start ();

		animationFight = new Animation (DiversLoader.getFightAnimation (), 0, 0, 8, 0, true, 160, true);
		animationFight.setCurrentFrame (8);
	}

	public void drawMenu () {
		// Fond
		MenuLoader.getHashMapImage ().get ("02").draw ();

		// Selection perso, spirit
		MenuLoader.getHashMapImage ().get ("16").draw ();
		MenuLoader.getHashMapImage ().get ("15").draw (37, 357 + (cursorChar * 56));

		// Vie et energie
		// du héro
		MenuLoader.getHashMapImage ().get ("24").draw (289, 382, (this.getController ().getModel ().getHero ().getHp () * 111 / this.getController ().getModel ().getHero ().getMaxHP ()), 7);
		// des esprits
		for (int i = 0; i < getController ().getModel ().getHero ().getListSpirit ().size (); i++) {
			MenuLoader.getHashMapImage ().get ("24").draw (289, 438 + (i * 56), (this.getController ().getModel ().getHero ().getListSpirit ().get (i).getHp () * 111 / this.getController ().getModel ().getHero ().getListSpirit ().get (i).getMaxHP ()), 7);
			for (int j = 0; j < this.getController ().getModel ().getHero ().getListSpirit ().get (i).getPower (); j++)
				MenuLoader.getHashMapImage ().get ("18").draw (330 + (j * 20), 417 + (i * 56));
		}

		// Monstre
		animationMonster.draw (550, 110);
		// Héro
		FontLoader.getFontShadow ().drawString (41, 365, this.getController ().getModel ().getHero ().getName ().split ("_")[0]);
		FontLoader.getFont ().drawString (40, 364, this.getController ().getModel ().getHero ().getName ().split ("_")[0]);
		animationHero.draw (50, 160);
		// Esprit
		for (int i = 0; i < this.getController ().getModel ().getHero ().getListSpirit ().size (); i++) {
			FontLoader.getFontShadow ().drawString (41, 421 + (i * 56), this.getController ().getModel ().getHero ().getListSpirit ().get (i).getName ().split ("_")[0]);
			FontLoader.getFont ().drawString (40, 420 + (i * 56), this.getController ().getModel ().getHero ().getListSpirit ().get (i).getName ().split ("_")[0]);
			animationSpirit.draw (130, 98);
		}

		// Affichage du jaillessement sanguin
		if (animationFight.getFrame () > 7) animationFight.stopAt (8);
		else animationFight.draw (this.getxFightAnim (), this.getyFightAnim ());

		// Selection Skill
		if (this.isUserToPlay () && this.isAttack ()) {
			MenuLoader.getHashMapImage ().get ("13").draw ();
			if (!this.isCharSelectionned ()) MenuLoader.getHashMapImage ().get ("14").draw (460, 332 + (cursorSkill * 56));
			switch (cursorChar) {
				case 0:
					int i = 0;
					for (Skill skill : listNameSkillsHero) {
						FontLoader.getFontShadow ().drawString (511, 341 + (i * 56), skill.getName ().split ("_")[0]);
						FontLoader.getFont ().drawString (510, 340 + (i * 56), skill.getName ().split ("_")[0]);

						i++;
					}
					break;
				case 1:
					int j = 0;
					for (Skill skill : this.getController ().getModel ().getHero ().getListSpirit ().get (0).getSkills ()) {
						FontLoader.getFontShadow ().drawString (511, 341 + (j * 56), skill.getName ().split ("_")[0]);
						FontLoader.getFont ().drawString (510, 340 + (j * 56), skill.getName ().split ("_")[0]);

						for (int k = 0; k < skill.getPointCost (); k++)
							if (k < this.getController ().getModel ().getHero ().getListSpirit ().get (0).getPower ()) MenuLoader.getHashMapImage ().get ("18").draw (460 + (k * 16), 360 + (j * 56));
							else MenuLoader.getHashMapImage ().get ("17").draw (460 + (k * 16), 360 + (j * 56));

						j++;
					}
					break;
				case 2:
					break;
				case 3:
					break;
				default:
					break;
			}
		}
		else {
			MenuLoader.getHashMapImage ().get ("25").draw ();
			FontLoader.getFontShadow ().drawString (500, 365, messageFight);
			FontLoader.getFont ().drawString (501, 364, messageFight);
			animationArrow.draw (497, 524);
		}
		if (affichMWin) MenuLoader.getHashMapImage ().get ("100").draw ();
	}

	public void upKey () {
		if (this.isCharSelectionned ()) {
			if (cursorChar - 1 >= 0) {
				cursorChar--;
				if (cursorChar == 0) attacking = this.getController ().getModel ().getHero ();
				else attacking = this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1);
			}
		}
		else {
			if (cursorSkill - 1 >= 0) cursorSkill--;
		}
	}

	public void downKey () {
		if (this.isCharSelectionned ()) {
			if (cursorChar + 1 <= nbSpirit) {
				cursorChar++;
				if (cursorChar == 0) attacking = this.getController ().getModel ().getHero ();
				else attacking = this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1);
			}
		}
		else {
			if (cursorChar == 0) {
				if (cursorSkill + 1 <= listArraySkills.get (cursorChar).size () - 1) cursorSkill++;
			}
			else {
				if (cursorSkill + 1 <= listArraySkills.get (cursorChar - 1).size () - 1) cursorSkill++;
			}
		}
	}

	public boolean heroAttack () {
		if (cursorChar != 0) {
			// s'il y a assez d'energie
			if (this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getSkills ().get (cursorSkill).getPointCost () <= this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getPower ()) {
				messageFight = BattleEngine.battleRound ((cursorChar == 0 ? listNameSkillsHero.get (cursorSkill) : this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getSkills ().get (cursorSkill)), attacking, monster);
				this.setxFightAnim (550);
				this.setyFightAnim (110);
				animationFight.restart ();
			}
			else return false;
		}
		else {
			messageFight = BattleEngine.battleRound ((cursorChar == 0 ? listNameSkillsHero.get (cursorSkill) : this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getSkills ().get (cursorSkill)), attacking, monster);
			this.setxFightAnim (550);
			this.setyFightAnim (110);
			animationFight.restart ();
		}
		if (cursorChar != 0) {
			int lostPoint = this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getSkills ().get (cursorSkill).getPointCost ();
			int power = this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).getPower ();
			if (power - lostPoint < 0) power = 0;
			else power -= lostPoint;
			this.getController ().getModel ().getHero ().getListSpirit ().get (cursorChar - 1).setPower (power);
		}
		if (this.getController ().getModel ().getHero ().getHp () < 1 || monster.getHp () < 1) {
			this.setEndFight (true);
			monster.setHp (monster.getMaxHP ());
			SoundsLoader.getFight ().stop ();
			affichMWin = true;
		}
		return true;
	}

	public void monsterAttack () {
		messageFight = monsterIA ();
		if (this.getController ().getModel ().getHero ().getHp () < 1) {
			monster.setHp (monster.getMaxHP ());
			SoundsLoader.getFight ().stop ();
			sbg.enterState (6, new FadeOutTransition (Color.black, 2000), new FadeInTransition (Color.black, 2000));
		}
	}

	public void attack () {
		if (this.isUserToPlay ()) {
			if (this.isAttack ()) {
				if (this.heroAttack ()) this.setAttack (false);
			}
			else {
				if (this.isEndDisplayHero ()) {
					this.setUserToPlay (false);
					this.setAttackM (true);
				}
				else {
					this.setEndDisplayHero (true);
				}
			}
		}
		else {
			if (this.isAttackM ()) {
				this.monsterAttack ();
				this.setAttackM (false);
				// On augment l'energie
				for (int i = 0; i < this.getController ().getModel ().getHero ().getListSpirit ().size (); i++) {
					int power = this.getController ().getModel ().getHero ().getListSpirit ().get (i).getPower ();
					if (power + 1 > 4) power = 4;
					else power++;
					this.getController ().getModel ().getHero ().getListSpirit ().get (i).setPower (power);
				}
			}
			else {
				if (this.isEndDisplayMonster ()) {
					this.setUserToPlay (true);
					this.setAttack (true);
				}
				else {
					this.setEndDisplayMonster (true);
				}
			}
		}
	}

	/**
	 * M�thode de comportement d'une IA
	 * 
	 * @return
	 */
	public String monsterIA () {
		String str = "";

		int nbOf = 1 + getController ().getModel ().getHero ().getListSpirit ().size ();

		int[] skillOrder = new int[16];
		int indexSkillOrder = 0;

		Random r = new Random ();
		int targetNumber = r.nextInt (nbOf);
		Skill usedSkill = null;

		for (int i = 0; i < monster.getChanceSkills ().length; i++) {
			for (int j = 0; j < monster.getChanceSkills ()[i]; j++) {
				skillOrder[indexSkillOrder] = i;
				indexSkillOrder++;
			}
		}

		int skillChoice = r.nextInt (indexSkillOrder);
		usedSkill = monster.getSkills ().get (skillOrder[skillChoice]);

		if (targetNumber == 0) {
			str += BattleEngine.battleRound (usedSkill, monster, this.getController ().getModel ().getHero ());
			this.setxFightAnim (40);
			this.setyFightAnim (130);
			animationFight.restart ();
		}
		else if (getController ().getModel ().getHero ().getListSpirit ().get (targetNumber - 1).getHp () > 0) {
			str += BattleEngine.battleRound (usedSkill, monster, this.getController ().getModel ().getHero ().getListSpirit ().get (targetNumber - 1));
			this.setxFightAnim (130);
			this.setyFightAnim (98);
			animationFight.restart ();
		}
		else {
			str += BattleEngine.battleRound (usedSkill, monster, this.getController ().getModel ().getHero ());
			this.setxFightAnim (40);
			this.setyFightAnim (130);
			animationFight.restart ();
		}

		return str;
	}

	/**
	 * @param cursorChar
	 *            the cursorChar to set
	 */
	public void setCursorChar (int cursorChar) {
		this.cursorChar = cursorChar;
	}

	/**
	 * @return the cursorChar
	 */
	public int getCursorChar () {
		return cursorChar;
	}

	/**
	 * @param cursorSkill
	 *            the cursorSkill to set
	 */
	public void setCursorSkill (int cursorSkill) {
		this.cursorSkill = cursorSkill;
	}

	/**
	 * @return the cursorSkill
	 */
	public int getCursorSkill () {
		return cursorSkill;
	}

	/**
	 * @param nbSpirit
	 *            the nbSpirit to set
	 */
	public void setNbSpirit (int nbSpirit) {
		this.nbSpirit = nbSpirit;
	}

	/**
	 * @return the nbSpirit
	 */
	public int getNbSpirit () {
		return nbSpirit;
	}

	/**
	 * @param charSelectionned
	 *            the charSelectionned to set
	 */
	public void setCharSelectionned (boolean charSelectionned) {
		this.charSelectionned = charSelectionned;
	}

	/**
	 * @return the charSelectionned
	 */
	public boolean isCharSelectionned () {
		return charSelectionned;
	}

	/**
	 * @return the messageFight
	 */
	public String getMessageFight () {
		return messageFight;
	}

	/**
	 * @param messageFight
	 *            the messageFight to set
	 */
	public void setMessageFight (String messageFight) {
		this.messageFight = messageFight;
	}

	/**
	 * @param userToPlay
	 *            the userToPlay to set
	 */
	public void setUserToPlay (boolean userToPlay) {
		this.userToPlay = userToPlay;
	}

	/**
	 * @return the userToPlay
	 */
	public boolean isUserToPlay () {
		return userToPlay;
	}

	/**
	 * @return the endFight
	 */
	public boolean isEndFight () {
		return endFight;
	}

	/**
	 * @param endFight
	 *            the endFight to set
	 */
	public void setEndFight (boolean endFight) {
		this.endFight = endFight;
	}

	/**
	 * @return the endDisplayMonster
	 */
	public boolean isEndDisplayMonster () {
		return endDisplayMonster;
	}

	/**
	 * @param endDisplayMonster
	 *            the endDisplayMonster to set
	 */
	public void setEndDisplayMonster (boolean endDisplayMonster) {
		this.endDisplayMonster = endDisplayMonster;
	}

	/**
	 * @return the endDisplayHero
	 */
	public boolean isEndDisplayHero () {
		return endDisplayHero;
	}

	/**
	 * @param endDisplayHero
	 *            the endDisplayHero to set
	 */
	public void setEndDisplayHero (boolean endDisplayHero) {
		this.endDisplayHero = endDisplayHero;
	}

	/**
	 * @return the attack
	 */
	public boolean isAttack () {
		return attack;
	}

	/**
	 * @param attack
	 *            the attack to set
	 */
	public void setAttack (boolean attack) {
		this.attack = attack;
	}

	/**
	 * @return the attackM
	 */
	public boolean isAttackM () {
		return attackM;
	}

	/**
	 * @param attackM
	 *            the attackM to set
	 */
	public void setAttackM (boolean attackM) {
		this.attackM = attackM;
	}

	/**
	 * @return the animationHero
	 */
	public Animation getAnimationHero () {
		return animationHero;
	}

	/**
	 * @param animationHero
	 *            the animationHero to set
	 */
	public void setAnimationHero (Animation animationHero) {
		this.animationHero = animationHero;
	}

	/**
	 * @return the attacking
	 */
	public Character getAttacking () {
		return attacking;
	}

	/**
	 * @param attacking
	 *            the attacking to set
	 */
	public void setAttacking (Character attacking) {
		this.attacking = attacking;
	}

	/**
	 * @return the animationSpirit
	 */
	public Animation getAnimationSpirit () {
		return animationSpirit;
	}

	/**
	 * @param animationSpirit
	 *            the animationSpirit to set
	 */
	public void setAnimationSpirit (Animation animationSpirit) {
		this.animationSpirit = animationSpirit;
	}

	/**
	 * @param animatinFight
	 *            the animatinFight to set
	 */
	public void setAnimationFight (Animation animatinFight) {
		this.animationFight = animatinFight;
	}

	/**
	 * @return the animatinFight
	 */
	public Animation getAnimationFight () {
		return animationFight;
	}

	/**
	 * @param xFightAnim
	 *            the xFightAnim to set
	 */
	public void setxFightAnim (int xFightAnim) {
		this.xFightAnim = xFightAnim;
	}

	/**
	 * @return the xFightAnim
	 */
	public int getxFightAnim () {
		return xFightAnim;
	}

	/**
	 * @param yFightAnim
	 *            the yFightAnim to set
	 */
	public void setyFightAnim (int yFightAnim) {
		this.yFightAnim = yFightAnim;
	}

	/**
	 * @return the yFightAnim
	 */
	public int getyFightAnim () {
		return yFightAnim;
	}

	/**
	 * @param keyNumber
	 *            the keyNumber to set
	 */
	public void setKeyNumber (String keyNumber) {
		this.keyNumber = keyNumber;
	}

	/**
	 * @return the keyNumber
	 */
	public String getKeyNumber () {
		return keyNumber;
	}

	/**
	 * @param affichMWin
	 *            the affichMWin to set
	 */
	public void setAffichMWin (boolean affichMWin) {
		this.affichMWin = affichMWin;
	}

	/**
	 * @return the affichMWin
	 */
	public boolean isAffichMWin () {
		return affichMWin;
	}
}
