// ////////////////////////////////////////////////////////////////////////
// Copyright © 2011 Jérémy Chevrier, Hugo Buffet, Jérémy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////
/**
 * 
 */
package model.menu;

import java.util.ArrayList;

import org.newdawn.slick.Animation;

import controller.Controller;

import loader.FontLoader;
import loader.events.EventXMLLoader;
import loader.menus.AvatarLoader;
import loader.menus.MenuLoader;
import model.events.NpcEvent;
import model.various.Message;

/**
 * @author Jérémy Chevrier
 * @version 0.2.1 : 2011-02-02, 6:05pm
 */
public class NpcMenu extends Menu {

	///////////////
	// Attributs //
	///////////////
	/**
	 * La liste des message du PNJ
	 */
	private ArrayList<Message> listMessage;
	/**
	 * Le compteur pour savoir où on est sur la liste des messages
	 */
	private int inc = 0;
	/**
	 * Le nombre de message du menu (et donc la fin)
	 */
	private int endMenu = 0;
	/**
	 * AnimationMap de la fleche
	 */
	private Animation animationArrow;
	/**
	 * Pour éviter de faire les changements de variable trop souvent (utile
	 * surtout pour les ajouts)
	 */
	private boolean see;
	/**
	 * 
	 */
	private String key;
	/**
	 * Si on peut incrémenter ou non (seulement lors de l'appuie de touche
	 */
	private boolean incrementable;

	//////////////////
	// Constructeur //
	//////////////////
	/**
	 * @param key
	 *            La clé de l'évènement
	 */
	public NpcMenu (String key, int[] gameVar, Controller controller) {
		super (key, gameVar, controller);
		this.setListMessage (((NpcEvent) EventXMLLoader.getHashMapEvent ().get (key)).getListMessage ());
		this.loadAnimations ();
		animationArrow.start ();
		this.setKey (key);
		this.setSee (false);
		this.setIncrementable (true);
	}

	//////////////
	// Méthodes //
	//////////////
	public void drawMenu () {
		MenuLoader.getHashMapImage ().get ("00").draw (63, 370); // On affiche les fonds
		MenuLoader.getHashMapImage ().get ("01").draw (63, 416);
		AvatarLoader.getHashMapImage ().get (this.getName ()).draw (67, 374); // On dessine l'avatar
		FontLoader.getFontShadow ().drawString (113, 375, this.getName ()); // On dessine le nom
		FontLoader.getFont ().drawString (112, 374, this.getName ());
		if (this.getListMessage ().get (inc).getIndex () == 0) {
			String[] polyLineMes = this.getListMessage ().get (inc).getMessage ().split ("\n");
			for (int i = 0; i < polyLineMes.length; i++) {
				FontLoader.getFontShadow ().drawString (82, 437 + (i * 30), polyLineMes[i]);
				FontLoader.getFont ().drawString (81, 436 + (i * 30), polyLineMes[i]);
			}
		}
		else {
			if (this.getListMessage ().get (inc).getValue () == this.getGameVar ()[this.getListMessage ().get (inc).getIndex ()]) {
				String[] polyLineMes = this.getListMessage ().get (inc).getMessage ().split ("\n");
				for (int i = 0; i < polyLineMes.length; i++) {
					FontLoader.getFontShadow ().drawString (82, 437 + (i * 30), polyLineMes[i]);
					FontLoader.getFont ().drawString (81, 436 + (i * 30), polyLineMes[i]);
				}
				if (!this.isSee ()) {
					// S'il y a des valeurs à changer
					if (this.getListMessage ().get (inc).getIndexEdit () != 0) {
						// Money
						if (this.getListMessage ().get (inc).getIndexEdit () == 1) {
							this.getController ().getModel ().getHero ().setMoney (this.getController ().getModel ().getHero ().getMoney () + this.getListMessage ().get (inc).getValueEditInt ());
							if (this.getListMessage ().get (inc).getValeuEditType ().equalsIgnoreCase ("Add")) {
								int[] tmp = this.getController ().getModel ().getGameVar ();
								tmp[this.getListMessage ().get (inc).getIndexEdit ()] += this.getListMessage ().get (inc).getValueEditInt ();
								this.getController ().getModel ().setGameVar (tmp);
							}
							else if (this.getListMessage ().get (inc).getValeuEditType ().equalsIgnoreCase ("Equal")) {
								int[] tmp = this.getController ().getModel ().getGameVar ();
								tmp[this.getListMessage ().get (inc).getIndexEdit ()] = this.getListMessage ().get (inc).getValueEditInt ();
								this.getController ().getModel ().setGameVar (tmp);
							}
						}
						// Variable globale à changer
						if (this.getListMessage ().get (inc).getIndexEdit () > 50) {
							if (this.getListMessage ().get (inc).getValeuEditType ().equalsIgnoreCase ("Add")) {
								int[] tmp = this.getController ().getModel ().getGameVar ();
								tmp[this.getListMessage ().get (inc).getIndexEdit ()] += this.getListMessage ().get (inc).getValueEditInt ();
								this.getController ().getModel ().setGameVar (tmp);
								String keyTmp = ((NpcEvent) EventXMLLoader.getHashMapEvent ().get (this.getKey ())).getKeyValueEvent ();
								if (!keyTmp.equals ("")) {
									if (((NpcEvent) EventXMLLoader.getHashMapEvent ().get (keyTmp)).getValueArrayIndex () > 9000 && this.getController ().getModel ().getGameVar ()[this.getListMessage ().get (inc).getIndexEdit ()] >= 1) {
										EventXMLLoader.getHashMapEvent ().get (keyTmp).setActived (false);
									}
								}
								this.setIncrementable (false);
							}
							else if (this.getListMessage ().get (inc).getValeuEditType ().equalsIgnoreCase ("Equal")) {
								int[] tmp = this.getController ().getModel ().getGameVar ();
								tmp[this.getListMessage ().get (inc).getIndexEdit ()] = this.getListMessage ().get (inc).getValueEditInt ();
								this.getController ().getModel ().setGameVar (tmp);
								String keyTmp = ((NpcEvent) EventXMLLoader.getHashMapEvent ().get (this.getKey ())).getKeyValueEvent ();
								if (!keyTmp.equals ("")) {
									if (((NpcEvent) EventXMLLoader.getHashMapEvent ().get (keyTmp)).getValueArrayIndex () > 9000 && this.getController ().getModel ().getGameVar ()[this.getListMessage ().get (inc).getIndexEdit ()] >= 1) {
										EventXMLLoader.getHashMapEvent ().get (keyTmp).setActived (false);
									}
								}
								this.setIncrementable (false);
							}
						}
					}
				}
				//				System.out.println ("Money du perso : " + this.getController ().getModel ().getHero ().getMoney ());
				this.setSee (true);
			}
			else {
				if (this.isIncrementable ()) {
					while (this.getListMessage ().get (inc).getValue () != this.getGameVar ()[this.getListMessage ().get (inc).getIndex ()]) {
						inc++;
						System.out.println (inc);
						if (this.getInc () >= this.getListMessage ().size ()) {
							this.getController ().getModel ().setOutMap (false);
							break;
						}
					}
				}
				else {
					String[] polyLineMes = this.getListMessage ().get (inc).getMessage ().split ("\n");
					for (int i = 0; i < polyLineMes.length; i++) {
						FontLoader.getFontShadow ().drawString (82, 437 + (i * 30), polyLineMes[i]);
						FontLoader.getFont ().drawString (81, 436 + (i * 30), polyLineMes[i]);
					}
				}
			}
		}

		animationArrow.draw (397, 524);
	}

	/**
	 * Charge toutes les animations du personnage
	 */
	private void loadAnimations () {
		// On charge la liste des animations
		animationArrow = new Animation (MenuLoader.getHashMapSprite ().get ("00"), 0, 0, 2, 0, true, 200, true);
	}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @param listMessage
	 *            the listMessage to set
	 */
	public void setListMessage (ArrayList<Message> listMessage) {
		this.listMessage = listMessage;
		for (Message msg : listMessage) {
			if (msg.getMessage ().contains ("#")) {
				String str = "#";
				int nb = Integer.parseInt (msg.getMessage ().split ("#")[1]);
				str += nb;
				str += "#";
				msg.setMessage (msg.getMessage ().replaceAll (str, String.valueOf (this.getController ().getModel ().getGameVar ()[nb])));
			}
		}
	}

	/**
	 * @return the listMessage
	 */
	public ArrayList<Message> getListMessage () {
		return listMessage;
	}

	/**
	 * @param endMenu
	 *            the endMenu to set
	 */
	public void setEndMenu (int endMenu) {
		this.endMenu = endMenu;
	}

	/**
	 * @return the endMenu
	 */
	public int getEndMenu () {
		return endMenu;
	}

	/**
	 * @return the inc
	 */
	public int getInc () {
		return inc;
	}

	/**
	 * @param inc
	 *            the inc to set
	 */
	public void setInc (int inc) {
		this.inc = inc;
	}

	/**
	 * @param see
	 *            the see to set
	 */
	public void setSee (boolean see) {
		this.see = see;
	}

	/**
	 * @return the see
	 */
	public boolean isSee () {
		return see;
	}

	/**
	 * @return the key
	 */
	public String getKey () {
		return key;
	}

	/**
	 * @param key
	 *            the key to set
	 */
	public void setKey (String key) {
		this.key = key;
	}

	/**
	 * @param incrementable
	 *            the incrementable to set
	 */
	public void setIncrementable (boolean incrementable) {
		this.incrementable = incrementable;
	}

	/**
	 * @return the incrementable
	 */
	public boolean isIncrementable () {
		return incrementable;
	}
}
