// ////////////////////////////////////////////////////////////////////////
// Copyright  2011 Jrmy Chevrier, Hugo Buffet, Jrmy Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details. //
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

package modules;

import java.io.FileInputStream;
import java.io.IOException;

import org.jdom.Document;
import org.jdom.JDOMException;
import org.jdom.input.SAXBuilder;
import org.newdawn.slick.Image;

/**
 * Classe Abstraite gnrique des lecteurs des fichiers xml. Comprend les
 * mthodes que utilisent ou doivent utiliser les diffrents lecteurs.
 * 
 * @author Hugo Buffet
 * @version 0.1.1 : 2011-01-20, 10:19am</br>
 * @version 0.1.2 : 2011-01-20, 6:54pm</br>
 * 
 */
public abstract class Reader {

	/**
	 * Mthode qui convertit le fichier en une instance Document afin de pouvoir
	 * le traiter.
	 * 
	 * @param fileName
	 *            url du fichier xml  traiter
	 * @return le fichier sous forme Document
	 * 
	 */
	public Document xmlReader (String fileName) {

		Document doc = null;
		// Cration d'un parseur SAX pour JDOM
		SAXBuilder builder = new SAXBuilder ();
		try {
			// Lecture du document XML et gnration d'un document JDOM
			doc = builder.build (new FileInputStream (fileName));
		}
		catch (JDOMException e) {
			System.out.println ("JDOM Parsing Error : " + e);
		}
		catch (IOException e2) {
			System.out.println ("Input Output Error : " + e2);
		}

		return doc;
	}

	/**
	 * Permet de rcuprer l'image du GameObject dans la liste des images
	 * prcharges.
	 * 
	 * @param keyImg
	 *            Identifiant de l'image associe au GameObject
	 * 
	 * @return l'image  l'emplacement de l'url
	 * 
	 */
	public abstract Image imageLoader (String keyImg);

	/**
	 * Mthode permettant de crer une instance d'un objet de jeu  partir de
	 * son fichier xml correspondant.
	 * 
	 * @param urlFile
	 *            url du fichier xml  traiter
	 * @return une instance de l'objet de jeu dcrit par le fichier xml
	 * 
	 */
	public abstract model.GameObject getGameObject (String urlFile);

}
