// ////////////////////////////////////////////////////////////////////////
// Copyright � 2011 J�r�my Chevrier, Hugo Buffet, J�r�my Lebrun
//
// This file is part of JZunBeHa.
//
// JZunBeHa is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// JZunBeHa is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
// GNU General Public License for more details. //
// You should have received a copy of the GNU General Public License
// along with JZunBeHa. If not, see <http://www.gnu.org/licenses/>.
// ////////////////////////////////////////////////////////////////////////

package tools.graphics;

import javax.swing.JFrame;
import javax.swing.JPanel;

import java.awt.BorderLayout;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.Font;
import java.awt.FontFormatException;

import javax.swing.BorderFactory;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JMenuBar;
import javax.swing.JMenu;
import javax.swing.JMenuItem;
import javax.swing.JLabel;
import javax.swing.JOptionPane;
import javax.swing.JSeparator;
import javax.swing.JTextArea;
import javax.swing.KeyStroke;
import javax.swing.SwingUtilities;
import javax.swing.UIManager;

import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.io.File;
import java.io.FileInputStream;
import java.io.IOException;

import javax.swing.JTextField;

import model.events.Event;
import modules.EventReader;
import tools.writers.EventWriter;

@SuppressWarnings("serial")
public class EventMaker extends JFrame implements ActionListener, KeyListener {

	private JPanel container = new JPanel ();
	private JMenu menuFile = null;
	private JMenuBar menu;
	private JMenuItem menuFileNew, menuFileOpen, menuFileSave, menuFileQuit;
	private JTextField textFieldKey, textFieldName, textFieldType;
	private JTextArea textFieldDesc;
	private JButton buttonSave;
	private String title = "EventMaker v1.0";
	private String file = "";
	private String editFile = "";
	private boolean newFile = true;
	private boolean edit = false;

	private Event event = null;
	private EventWriter eventWriter = new EventWriter ();
	private EventReader eventReader = new EventReader ();

	public EventMaker () {
		super ();
		this.setSize (new Dimension (1024, 768));
		this.setTitle (title);
		this.setDefaultCloseOperation (JFrame.EXIT_ON_CLOSE);
		container.setLayout (new BoxLayout (container, BoxLayout.PAGE_AXIS));
		container.setBorder (BorderFactory.createTitledBorder ("Information event"));
		this.setContentPane (container);

		// On ajoute le menu (et ses différents sous-menus)
		this.ajouterMenuBar ();
		// On ajoute les panneaux
		this.initComponents ();

		this.nouveau ();

		this.setNimbusLook ();
		this.pack ();
		this.setVisible (true);
	}

	private void initComponents () {
		// Initialisation
		// Partie Key
		JPanel panelLabelKey = new JPanel (new FlowLayout (FlowLayout.LEFT));
		JLabel labelKey = new JLabel ("Key :");
		panelLabelKey.add (labelKey);

		JPanel panelTextFieldKey = new JPanel (new FlowLayout (FlowLayout.RIGHT));
		textFieldKey = new JTextField (20);
		textFieldKey.addKeyListener (this);
		panelTextFieldKey.add (textFieldKey);

		JPanel panelKey = new JPanel (new BorderLayout ());
		panelKey.add (panelLabelKey, BorderLayout.WEST);
		panelKey.add (panelTextFieldKey, BorderLayout.CENTER);

		// Partie Name
		JPanel panelLabelName = new JPanel (new FlowLayout (FlowLayout.LEFT));
		JLabel labelName = new JLabel ("Name :");
		panelLabelName.add (labelName);

		JPanel panelTextFieldName = new JPanel (new FlowLayout (FlowLayout.RIGHT));
		textFieldName = new JTextField (20);
		textFieldName.addKeyListener (this);
		panelTextFieldName.add (textFieldName);

		JPanel panelName = new JPanel (new BorderLayout ());
		panelName.add (panelLabelName, BorderLayout.WEST);
		panelName.add (panelTextFieldName, BorderLayout.CENTER);

		// Partie Type
		JPanel panelLabelType = new JPanel (new FlowLayout (FlowLayout.LEFT));
		JLabel labelType = new JLabel ("Type :");
		panelLabelType.add (labelType);

		JPanel panelTextFieldType = new JPanel (new FlowLayout (FlowLayout.RIGHT));
		textFieldType = new JTextField (20);
		textFieldType.addKeyListener (this);
		panelTextFieldType.add (textFieldType);

		JPanel panelType = new JPanel (new BorderLayout ());
		panelType.add (panelLabelType, BorderLayout.WEST);
		panelType.add (panelTextFieldType, BorderLayout.CENTER);

		// Partie Description
		JPanel panelLabelDesc = new JPanel (new FlowLayout (FlowLayout.LEFT));
		JLabel labelDesc = new JLabel ("Description :");
		panelLabelDesc.add (labelDesc);

		JPanel panelTextFieldDesc = new JPanel (new FlowLayout (FlowLayout.LEFT));
		textFieldDesc = new JTextArea ();
		textFieldDesc.setLineWrap (true);
		textFieldDesc.setWrapStyleWord (true);
		textFieldDesc.addKeyListener (this);
		textFieldDesc.setPreferredSize (new Dimension (660, 120));
		Font font = null;
		try {
			font = new Font (Font.createFont (Font.TRUETYPE_FONT, new FileInputStream (new File ("data/ressources/fonts/AMERSN__.ttf"))).getFamily (), Font.PLAIN, 20);
		}
		catch (FontFormatException e) {
			e.printStackTrace ();
		}
		catch (IOException e) {
			e.printStackTrace ();
		}
		textFieldDesc.setFont (font);
		panelTextFieldDesc.add (textFieldDesc);

		JPanel panelDesc = new JPanel (new BorderLayout ());
		panelDesc.add (panelLabelDesc, BorderLayout.NORTH);
		panelDesc.add (panelTextFieldDesc, BorderLayout.SOUTH);

		// Partie Button
		JPanel panelButtonSave = new JPanel (new FlowLayout (FlowLayout.RIGHT));
		buttonSave = new JButton ("Save");
		buttonSave.addActionListener (this);
		panelButtonSave.add (buttonSave);

		// Partie regroupement
		container.add (panelKey);
		container.add (panelName);
		container.add (panelType);
		container.add (new JSeparator (JSeparator.HORIZONTAL));
		container.add (panelDesc);
		container.add (panelButtonSave);
	}

	private void ajouterMenuBar () {
		// Initialisation de la barre de menu
		menu = new JMenuBar ();

		// Initialisation des différents menu
		menuFile = new JMenu ("File");

		// Mnémonics
		menuFile.setMnemonic (KeyEvent.VK_F);

		// Les items des menus
		menuFileNew = new JMenuItem ("New");
		menuFileOpen = new JMenuItem ("Open");
		menuFileSave = new JMenuItem ("Save");
		menuFileQuit = new JMenuItem ("Quit");

		// Accélérateurs
		menuFileNew.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_N, KeyEvent.CTRL_DOWN_MASK));
		menuFileOpen.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_O, KeyEvent.CTRL_DOWN_MASK));
		menuFileSave.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_S, KeyEvent.CTRL_DOWN_MASK));
		menuFileQuit.setAccelerator (KeyStroke.getKeyStroke (KeyEvent.VK_Q, KeyEvent.CTRL_DOWN_MASK));

		// Listeners
		menuFileNew.addActionListener (this);
		menuFileOpen.addActionListener (this);
		menuFileSave.addActionListener (this);
		menuFileQuit.addActionListener (this);

		// On fixe la barre de menu et on lui ajoute tous les menus et items
		this.setJMenuBar (menu);
		menu.add (menuFile);
		menuFile.add (menuFileNew);
		menuFile.add (menuFileOpen);
		menuFile.add (menuFileSave);
		menuFile.add (menuFileQuit);
	}

	private void setNimbusLook () {
		// Look Nimbus, futur LAF multi-platforme par défaut (JDK 7) et franchement il est largement mieux que l'actuel (Metal)
		for (UIManager.LookAndFeelInfo laf : UIManager.getInstalledLookAndFeels ()) {
			if ("Nimbus".equals (laf.getName ())) {
				try {
					UIManager.setLookAndFeel (laf.getClassName ());
					SwingUtilities.updateComponentTreeUI (this);
				}
				catch (Exception e) {}
			}
		}
	}

	/**
	 * 
	 */
	//m�thodes
	public void nouveau () {
		this.event = new Event ("", null, "", 0, 0, "", 0, "");

		this.textFieldKey.setText ("");
		this.textFieldName.setText ("");
		this.textFieldType.setText ("");
		this.textFieldDesc.setText ("");
	}

	public void fileChooser () {
		JFileChooser openFile = new JFileChooser (".");
		int returnValeur = openFile.showOpenDialog (this);
		if (returnValeur == JFileChooser.APPROVE_OPTION) this.ouvrir (openFile.getSelectedFile ().getAbsolutePath ());
	}

	public void ouvrir (String urlFile) {
		this.event = (Event) this.eventReader.getGameObject (urlFile);
		remplirChamps ();
		String[] strTmp = urlFile.split ("/");
		file = " - " + strTmp[strTmp.length-1];
		editFile = "";
		this.setTitle (title + file + editFile);
	}

	public void appliquer () {
		this.event.setKey (this.textFieldKey.getText ());
		this.event.setName (this.textFieldName.getText ());
		this.event.setEventType (this.textFieldType.getText ());
		this.event.setDescription (this.textFieldDesc.getText ());
	}

	public void sauvegarder () {
		file = " - " + this.eventWriter.writeGM (this.event);
		editFile = "";
		this.setTitle (title + file + editFile);
	}

	public void remplirChamps () {
		this.textFieldKey.setText (this.event.getKey ());
		this.textFieldName.setText (this.event.getName ());
		this.textFieldType.setText (this.event.getEventType ());
		this.textFieldDesc.setText (this.event.getDescription ());
	}

	public void save () {
		this.appliquer ();
		this.sauvegarder ();
		JOptionPane.showMessageDialog (null, "Le fichier a bien �t� sauvegard�.", "Confirmation", JOptionPane.INFORMATION_MESSAGE);
	}

	public void newF () {
		this.nouveau ();
		file = " - New File";
		editFile = "";
		this.setTitle (title + file + editFile);
	}

	public void actionPerformed (ActionEvent arg0) {
		if (arg0.getSource ().equals (this.menuFileNew)) {
			this.newF ();
		}
		else if (arg0.getSource ().equals (this.menuFileOpen)) {
			this.fileChooser ();
		}
		else if (arg0.getSource ().equals (this.menuFileSave)) {
			this.save ();
		}
		else if (arg0.getSource () == this.buttonSave) {
			this.save ();
		}
		else if (arg0.getSource ().equals (this.menuFileQuit)) {
			this.dispose ();
		}
	}

	/**
	 * @param args
	 */
	public static void main (String[] args) {
		new EventMaker ();
	}

	@Override
	public void keyPressed (KeyEvent arg0) {
		if (this.isNewFile ()) file = " - New File";
		editFile = " *";
		this.setTitle (title + file + editFile);
	}

	@Override
	public void keyReleased (KeyEvent arg0) {}

	@Override
	public void keyTyped (KeyEvent arg0) {}

	////////////////
	// Accesseurs //
	////////////////
	/**
	 * @return the newFile
	 */
	public boolean isNewFile () {
		return newFile;
	}

	/**
	 * @param newFile
	 *            the newFile to set
	 */
	public void setNewFile (boolean newFile) {
		this.newFile = newFile;
	}

	/**
	 * @param edit the edit to set
	 */
	public void setEdit (boolean edit) {
		this.edit = edit;
	}

	/**
	 * @return the edit
	 */
	public boolean isEdit () {
		return edit;
	}
}
